"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import requests as requests_http
from ...models.components import createmodelevent as components_createmodelevent
from ...models.components import sessionpropertiesbatch as components_sessionpropertiesbatch
from dataclasses_json import Undefined, dataclass_json
from honeyhive import utils
from typing import List, Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class CreateModelEventBatchRequestBody:
    model_events: Optional[List[components_createmodelevent.CreateModelEvent]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('model_events'), 'exclude': lambda f: f is None }})
    is_single_session: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('is_single_session'), 'exclude': lambda f: f is None }})
    r"""Default is false. If true, all events will be associated with the same session"""
    session_properties: Optional[components_sessionpropertiesbatch.SessionPropertiesBatch] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('session_properties'), 'exclude': lambda f: f is None }})
    



@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class CreateModelEventBatchResponseBody:
    r"""Model events created"""
    event_ids: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('event_ids'), 'exclude': lambda f: f is None }})
    success: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('success'), 'exclude': lambda f: f is None }})
    



@dataclasses.dataclass
class CreateModelEventBatchResponse:
    content_type: str = dataclasses.field()
    r"""HTTP response content type for this operation"""
    status_code: int = dataclasses.field()
    r"""HTTP response status code for this operation"""
    raw_response: requests_http.Response = dataclasses.field()
    r"""Raw HTTP response; suitable for custom response parsing"""
    object: Optional[CreateModelEventBatchResponseBody] = dataclasses.field(default=None)
    r"""Model events created"""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import requests as requests_http
from ...models.components import metric as components_metric
from typing import List, Optional


@dataclasses.dataclass
class GetMetricsRequest:
    project_name: str = dataclasses.field(metadata={'query_param': { 'field_name': 'project_name', 'style': 'form', 'explode': True }})
    r"""Project name associated with metrics"""
    



@dataclasses.dataclass
class GetMetricsResponse:
    content_type: str = dataclasses.field()
    r"""HTTP response content type for this operation"""
    status_code: int = dataclasses.field()
    r"""HTTP response status code for this operation"""
    raw_response: requests_http.Response = dataclasses.field()
    r"""Raw HTTP response; suitable for custom response parsing"""
    metrics: Optional[List[components_metric.Metric]] = dataclasses.field(default=None)
    r"""A list of metrics"""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from honeyhive import utils
from typing import Any, Dict, Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class UpdateRunResponse:
    UNSET='__SPEAKEASY_UNSET__'
    evaluation: Optional[Dict[str, Any]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('evaluation'), 'exclude': lambda f: f is None }})
    r"""Database update success message"""
    warning: Optional[str] = dataclasses.field(default=UNSET, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('warning'), 'exclude': lambda f: f is UpdateRunResponse.UNSET }})
    r"""A warning message if the logged events don't have an associated datapoint id on the event metadata"""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import requests as requests_http
from ...models.components import updaterunrequest as components_updaterunrequest
from ...models.components import updaterunresponse as components_updaterunresponse
from typing import Optional


@dataclasses.dataclass
class UpdateRunRequest:
    run_id: str = dataclasses.field(metadata={'path_param': { 'field_name': 'run_id', 'style': 'simple', 'explode': False }})
    update_run_request: components_updaterunrequest.UpdateRunRequest = dataclasses.field(metadata={'request': { 'media_type': 'application/json' }})
    



@dataclasses.dataclass
class UpdateRunResponse:
    content_type: str = dataclasses.field()
    r"""HTTP response content type for this operation"""
    status_code: int = dataclasses.field()
    r"""HTTP response status code for this operation"""
    raw_response: requests_http.Response = dataclasses.field()
    r"""Raw HTTP response; suitable for custom response parsing"""
    update_run_response: Optional[components_updaterunresponse.UpdateRunResponse] = dataclasses.field(default=None)
    r"""Successful response"""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import requests as requests_http
from ...models.components import configuration as components_configuration
from enum import Enum
from typing import List, Optional


class Env(str, Enum):
    r"""Environment - \\"dev\\", \\"staging\\" or \\"prod\\" """
    DEV = 'dev'
    STAGING = 'staging'
    PROD = 'prod'


@dataclasses.dataclass
class GetConfigurationsRequest:
    project: str = dataclasses.field(metadata={'query_param': { 'field_name': 'project', 'style': 'form', 'explode': True }})
    r"""Project name for configuration like `Example Project`"""
    env: Optional[Env] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'env', 'style': 'form', 'explode': True }})
    r"""Environment - \\"dev\\", \\"staging\\" or \\"prod\\" """
    name: Optional[str] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'name', 'style': 'form', 'explode': True }})
    r"""The name of the configuration like `v0`"""
    



@dataclasses.dataclass
class GetConfigurationsResponse:
    content_type: str = dataclasses.field()
    r"""HTTP response content type for this operation"""
    status_code: int = dataclasses.field()
    r"""HTTP response status code for this operation"""
    raw_response: requests_http.Response = dataclasses.field()
    r"""Raw HTTP response; suitable for custom response parsing"""
    configurations: Optional[List[components_configuration.Configuration]] = dataclasses.field(default=None)
    r"""An array of configurations"""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .adddatapoints import *
from .createconfiguration import *
from .createdatapoint import *
from .createdataset import *
from .createevent import *
from .createeventbatch import *
from .createmetric import *
from .createmodelevent import *
from .createmodeleventbatch import *
from .createproject import *
from .createrun import *
from .createtool import *
from .deleteconfiguration import *
from .deletedatapoint import *
from .deletedataset import *
from .deletemetric import *
from .deleteproject import *
from .deleterun import *
from .deletetool import *
from .getconfigurations import *
from .getdatapoint import *
from .getdatapoints import *
from .getdatasets import *
from .getevents import *
from .getexperimentcomparison import *
from .getexperimentresult import *
from .getmetrics import *
from .getprojects import *
from .getrun import *
from .getruns import *
from .getsession import *
from .gettools import *
from .startsession import *
from .updateconfiguration import *
from .updatedatapoint import *
from .updatedataset import *
from .updateevent import *
from .updatemetric import *
from .updateproject import *
from .updaterun import *
from .updatetool import *

__all__ = ["AddDatapointsRequest","AddDatapointsRequestBody","AddDatapointsResponse","AddDatapointsResponseBody","AggregateFunction","CreateConfigurationResponse","CreateDatapointResponse","CreateDatapointResponseBody","CreateDatapointResult","CreateDatasetResponse","CreateDatasetResponseBody","CreateDatasetResult","CreateEventBatchRequestBody","CreateEventBatchResponse","CreateEventBatchResponseBody","CreateEventRequestBody","CreateEventResponse","CreateEventResponseBody","CreateMetricResponse","CreateModelEventBatchRequestBody","CreateModelEventBatchResponse","CreateModelEventBatchResponseBody","CreateModelEventRequestBody","CreateModelEventResponse","CreateModelEventResponseBody","CreateProjectResponse","CreateRunResponse","CreateToolResponse","CreateToolResponseBody","DateRange","DeleteConfigurationRequest","DeleteConfigurationResponse","DeleteDatapointRequest","DeleteDatapointResponse","DeleteDatapointResponseBody","DeleteDatasetRequest","DeleteDatasetResponse","DeleteMetricRequest","DeleteMetricResponse","DeleteProjectRequest","DeleteProjectResponse","DeleteRunRequest","DeleteRunResponse","DeleteToolRequest","DeleteToolResponse","Env","GetConfigurationsRequest","GetConfigurationsResponse","GetDatapointRequest","GetDatapointResponse","GetDatapointResponseBody","GetDatapointsRequest","GetDatapointsResponse","GetDatapointsResponseBody","GetDatasetsRequest","GetDatasetsResponse","GetDatasetsResponseBody","GetEventsRequestBody","GetEventsResponse","GetEventsResponseBody","GetExperimentComparisonRequest","GetExperimentComparisonResponse","GetExperimentResultRequest","GetExperimentResultResponse","GetMetricsRequest","GetMetricsResponse","GetProjectsRequest","GetProjectsResponse","GetRunRequest","GetRunResponse","GetRunsRequest","GetRunsResponse","GetSessionRequest","GetSessionResponse","GetToolsResponse","Mapping","QueryParamAggregateFunction","Result","StartSessionRequestBody","StartSessionResponse","StartSessionResponseBody","Type","UpdateConfigurationRequest","UpdateConfigurationResponse","UpdateDatapointRequest","UpdateDatapointResponse","UpdateDatasetResponse","UpdateEventRequestBody","UpdateEventResponse","UpdateMetricResponse","UpdateProjectResponse","UpdateRunRequest","UpdateRunResponse","UpdateToolResponse"]



import os
import uuid
from pathlib import Path
import logging
import sys
import platform
import requests
from posthog import Posthog
import pkg_resources

POSTHOG_API_KEY = "phc_yeqaIP07fjwZ5n3w47wPtSz7G58igfczuQ9X3zKhuxa"

class Telemetry:
    ANON_ID_PATH = str(Path.home() / ".cache" / "honeyhive" / "telemetry_anon_id")
    UNKNOWN_ANON_ID = "UNKNOWN"
    _posthog: Posthog = None

    def __new__(cls) -> "Telemetry":
        if not hasattr(cls, "instance"):
            obj = cls.instance = super(Telemetry, cls).__new__(cls)
            obj._telemetry_enabled = (
                os.getenv("HONEYHIVE_TELEMETRY") or "true"
            ).lower() == "true" and "pytest" not in sys.modules
            if obj._telemetry_enabled:
                try:
                    obj._posthog = Posthog(
                        project_api_key=POSTHOG_API_KEY,
                        host="https://app.posthog.com",
                    )
                    obj._curr_anon_id = None
                    posthog_logger = logging.getLogger("posthog")
                    posthog_logger.disabled = True
                except Exception:
                    # disable telemetry if it fails
                    obj._telemetry_enabled = False
        return cls.instance

    def _anon_id(self) -> str:
        if self._curr_anon_id:
            return self._curr_anon_id
        try:
            if not os.path.exists(self.ANON_ID_PATH):
                os.makedirs(os.path.dirname(self.ANON_ID_PATH), exist_ok=True)
                with open(self.ANON_ID_PATH, "w") as f:
                    new_anon_id = str(uuid.uuid4())
                    f.write(new_anon_id)
                self._curr_anon_id = new_anon_id
            else:
                with open(self.ANON_ID_PATH, "r") as f:
                    self._curr_anon_id = f.read()
        except Exception:
            self._curr_anon_id = self.UNKNOWN_ANON_ID
        return self._curr_anon_id

    def _get_package_version(self, package_name: str) -> str:
        try:
            return pkg_resources.get_distribution(package_name).version
        except pkg_resources.DistributionNotFound:
            return "Not installed"

    def _context(self) -> dict:
        context = {
            "sdk": "honeyhive",
            "sdk_version": self._get_package_version("honeyhive"),
        }
        
        # Language settings
        context["language"] = {
            "name": "python",
            "version": platform.python_version(),
            "implementation": platform.python_implementation(),
        }
        
        # Runtime settings
        context["runtime"] = self._get_runtime_info()
        
        # OS version
        context["os"] = {
            "name": os.name,
            "system": platform.system(),
            "release": platform.release(),
            "version": platform.version(),
        }
        
        # Chip version
        context["chip"] = {
            "architecture": platform.machine(),
            "processor": platform.processor(),
        }

        # Service Provider Info
        context["service_provider_info"] = self._get_service_provider_info()

        # Notebook Environment Info
        context["notebook_info"] = self._get_notebook_info()

        # Execution Info
        context["execution_info"] = self._get_execution_info()
        
        # Package versions
        context["package_versions"] = {
            # orchestration frameworks
            "langchain": self._get_package_version("langchain"),
            "llamaindex": self._get_package_version("llama-index"),
            "haystack": self._get_package_version("haystack"),
            "boto3": self._get_package_version("boto3"),

            # llms
            "openai": self._get_package_version("openai"),
            "anthropic": self._get_package_version("anthropic"),
            "ollama": self._get_package_version("ollama"),
            "mistralai": self._get_package_version("mistralai"),
            "google-generativeai": self._get_package_version("google-generativeai"),
            "watsonx": self._get_package_version("ibm-watson-machine-learning"),
            "vertexai": self._get_package_version("google-cloud-aiplatform"),
            "transformers": self._get_package_version("transformers"),
            "together": self._get_package_version("together"),
            "replicate": self._get_package_version("replicate"),
            "groq": self._get_package_version("groq"),

            # vector dbs
            "qdrant": self._get_package_version("qdrant-client"),
            "weaviate": self._get_package_version("weaviate-client"),
            "milvus": self._get_package_version("pymilvus"),
            "pinecone": self._get_package_version("pinecone-client"),
        }
        
        return context

    def _get_runtime_info(self) -> dict:
        runtime_info = {}
        
        # Detect if running in a cloud environment
        if 'LAMBDA_TASK_ROOT' in os.environ:
            runtime_info['environment'] = 'AWS Lambda'
        elif 'RoleRoot' in os.environ:
            runtime_info['environment'] = 'Azure Functions'
        elif 'GOOGLE_CLOUD_PROJECT' in os.environ:
            runtime_info['environment'] = 'Google Cloud Functions'
        else:
            runtime_info['environment'] = 'Unknown'

        # Detect server-side framework
        try:
            import django
            runtime_info['framework'] = f"Django {django.get_version()}"
        except ImportError:
            try:
                import flask
                runtime_info['framework'] = f"Flask {flask.__version__}"
            except ImportError:
                try:
                    import fastapi
                    runtime_info['framework'] = f"FastAPI {fastapi.__version__}"
                except ImportError:
                    runtime_info['framework'] = "Unknown"

        return runtime_info

    def _get_service_provider_info(self) -> dict:
        try:
            response = requests.get('https://ipinfo.io/json', timeout=5)
            if response.status_code == 200:
                return response.json()
            else:
                return {'error': f'Failed to fetch data: HTTP {response.status_code}'}
        except Exception as e:
            return {'error': f'Failed to fetch data: {str(e)}'}

    def _get_notebook_info(self) -> dict:
        notebook_info = {}

        # Check if we're in a notebook environment
        notebook_info['in_notebook'] = 'ipykernel' in sys.modules

        # Get more detailed IPython information
        if notebook_info['in_notebook']:
            notebook_info['environment'] = 'notebook'
            try:
                import IPython
                ip_version = IPython.version_info
                notebook_info['ipython_version'] = '.'.join(map(str, ip_version))

                # Get client information
                ip = sys.modules['ipykernel']
                notebook_info['ipython_client'] = ip.write_connection_file.__module__.split('.')[0]

                # Get system info
                sys_info = IPython.utils.sysinfo.get_sys_info()
                notebook_info['sys_info'] = {
                    'ipython_version': sys_info['ipython_version'],
                    'ipython_path': sys_info['ipython_path']
                }
            except Exception as e:
                notebook_info['error'] = f"Failed to get detailed IPython info: {str(e)}"
        elif 'IPython' in sys.modules:
            notebook_info['environment'] = 'terminal'
        else:
            notebook_info['environment'] = 'standard_python'

        return notebook_info

    def _get_execution_info(self) -> dict:
        execution_info = {}
        
        # Get the Python executable path
        execution_info['executable'] = sys.executable
        
        # Get the command-line arguments
        execution_info['argv'] = sys.argv
        
        # Determine how the script was likely executed
        if sys.argv[0].endswith('pyinstaller-script.py'):
            execution_info['execution_method'] = 'pyinstaller'
        elif 'python' in sys.executable.lower():
            if len(sys.argv) > 0 and sys.argv[0].endswith('.py'):
                execution_info['execution_method'] = 'python_script'
            else:
                execution_info['execution_method'] = 'python_module'
        elif 'ipython' in sys.executable.lower():
            execution_info['execution_method'] = 'ipython'
        elif 'jupyter' in sys.executable.lower():
            execution_info['execution_method'] = 'jupyter'
        else:
            execution_info['execution_method'] = 'unknown'
        
        return execution_info

    def capture(self, event: str, event_properties: dict = {}) -> None:
        try:  # don't fail if telemetry fails
            if self._telemetry_enabled:
                self._posthog.capture(
                    self._anon_id(), event, {**self._context(), **event_properties}
                )
        except Exception:
            pass

    def log_exception(self, exception: Exception):
        try:  # don't fail if telemetry fails
            return self._posthog.capture(
                self._anon_id(),
                "exception",
                {
                    **self._context(),
                    "exception": str(exception),
                },
            )
        except Exception:
            pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .createeventbatch import *
from .createmodeleventbatch import *
from .sdkerror import *

__all__ = ["CreateEventBatchResponseBody","CreateModelEventBatchResponseBody","SDKError"]



import argparse
import os
import sys
import textwrap
import traceback

from honeyhive.cli import eval


def main(args=None):
    """The main routine."""

    # Add the current working directory to sys.path, similar to python's
    # unittesting frameworks.
    sys.path.insert(0, os.getcwd())

    if args is None:
        args = sys.argv[1:]

    parent_parser = argparse.ArgumentParser(add_help=False)
    parent_parser.add_argument(
        "--verbose",
        "-v",
        default=0,
        action="count",
        help="Include additional details, including full stack traces on errors. Pass twice (-vv) for debug logging.",
    )

    parser = argparse.ArgumentParser(
        description=textwrap.dedent(
            """honeyhive is a cli tool to work with HoneyHive.
    To see help for a specific subcommand, run `honeyhive <subcommand> --help`,
    e.g. `honeyhive eval --help`"""
        )
    )
    subparsers = parser.add_subparsers(help="sub-command help", dest="subcommand", required=True)

    for module in [eval]:
        module.build_parser(subparsers, parent_parser)

    args = parser.parse_args(args=args)
    # level = logging.DEBUG if args.verbose >= 2 else logging.INFO
    # logging.basicConfig(format="%(asctime)s %(levelname)s [%(name)s]: %(message)s", level=level)

    return args.func(args)


if __name__ == "__main__":
    try:
        ret = main()
        if ret:
            os._exit(1)
    except:
        traceback.print_exc()
        os._exit(1)


"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import requests as requests_http
from dataclasses_json import Undefined, dataclass_json
from honeyhive import utils
from typing import Any, Dict, List, Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Mapping:
    r"""Mapping of keys in the data object to be used as inputs, ground truth, and history, everything else goes into metadata"""
    inputs: List[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('inputs') }})
    r"""List of keys in the data object to be used as inputs"""
    ground_truth: List[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('ground_truth') }})
    r"""List of keys in the data object to be used as ground truth"""
    history: List[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('history') }})
    r"""List of keys in the data object to be used as chat history, can be empty list if not needed"""
    



@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class AddDatapointsRequestBody:
    project: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('project') }})
    r"""Name of the project associated with this dataset like `New Project`"""
    data: List[Dict[str, Any]] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('data') }})
    r"""List of JSON objects to be added as datapoints"""
    mapping: Mapping = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('mapping') }})
    r"""Mapping of keys in the data object to be used as inputs, ground truth, and history, everything else goes into metadata"""
    



@dataclasses.dataclass
class AddDatapointsRequest:
    dataset_id: str = dataclasses.field(metadata={'path_param': { 'field_name': 'dataset_id', 'style': 'simple', 'explode': False }})
    r"""The unique identifier of the dataset to add datapoints to like  `663876ec4611c47f4970f0c3`"""
    request_body: AddDatapointsRequestBody = dataclasses.field(metadata={'request': { 'media_type': 'application/json' }})
    



@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class AddDatapointsResponseBody:
    r"""Successful addition"""
    inserted: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('inserted'), 'exclude': lambda f: f is None }})
    datapoint_ids: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('datapoint_ids'), 'exclude': lambda f: f is None }})
    r"""List of unique datapoint ids added to the dataset"""
    



@dataclasses.dataclass
class AddDatapointsResponse:
    content_type: str = dataclasses.field()
    r"""HTTP response content type for this operation"""
    status_code: int = dataclasses.field()
    r"""HTTP response status code for this operation"""
    raw_response: requests_http.Response = dataclasses.field()
    r"""Raw HTTP response; suitable for custom response parsing"""
    object: Optional[AddDatapointsResponseBody] = dataclasses.field(default=None)
    r"""Successful addition"""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import requests as requests_http
from dataclasses_json import Undefined, dataclass_json
from honeyhive import utils
from typing import List, Optional


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class CreateEventBatchResponseBody(Exception):
    r"""Events partially created"""
    event_ids: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('event_ids'), 'exclude': lambda f: f is None }})
    errors: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('errors'), 'exclude': lambda f: f is None }})
    success: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('success'), 'exclude': lambda f: f is None }})
    raw_response: Optional[requests_http.Response] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'exclude': lambda f: True }})
    r"""Raw HTTP response; suitable for custom response parsing"""
    

    def __str__(self) -> str:
        return utils.marshal_json(self, type(self))



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

import requests as requests_http
from .sdkconfiguration import SDKConfiguration
from honeyhive import utils
from honeyhive._hooks import AfterErrorContext, AfterSuccessContext, BeforeRequestContext, HookContext
from honeyhive.models import errors, operations
from typing import Optional

class Events:
    sdk_configuration: SDKConfiguration

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        self.sdk_configuration = sdk_config
        
    
    
    def create_event(self, request: operations.CreateEventRequestBody) -> operations.CreateEventResponse:
        r"""Create a new event
        Please refer to our instrumentation guide for detailed information
        """
        hook_ctx = HookContext(operation_id='createEvent', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/events'
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        req_content_type, data, form = utils.serialize_request_body(request, operations.CreateEventRequestBody, "request", False, False, 'json')
        if req_content_type is not None and req_content_type not in ('multipart/form-data', 'multipart/mixed'):
            headers['content-type'] = req_content_type
        if data is None and form is None:
            raise Exception('request body is required')
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('POST', url, params=query_params, data=data, files=form, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        res = operations.CreateEventResponse(status_code=http_res.status_code, content_type=http_res.headers.get('Content-Type') or '', raw_response=http_res)
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[operations.CreateEventResponseBody])
                res.object = out
            else:
                content_type = http_res.headers.get('Content-Type')
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

        return res

    
    
    def update_event(self, request: operations.UpdateEventRequestBody) -> operations.UpdateEventResponse:
        r"""Update an event"""
        hook_ctx = HookContext(operation_id='updateEvent', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/events'
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        req_content_type, data, form = utils.serialize_request_body(request, operations.UpdateEventRequestBody, "request", False, False, 'json')
        if req_content_type is not None and req_content_type not in ('multipart/form-data', 'multipart/mixed'):
            headers['content-type'] = req_content_type
        if data is None and form is None:
            raise Exception('request body is required')
        headers['Accept'] = '*/*'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('PUT', url, params=query_params, data=data, files=form, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        res = operations.UpdateEventResponse(status_code=http_res.status_code, content_type=http_res.headers.get('Content-Type') or '', raw_response=http_res)
        
        if http_res.status_code == 200:
            pass
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

        return res

    
    
    def get_events(self, request: operations.GetEventsRequestBody) -> operations.GetEventsResponse:
        r"""Retrieve events based on filters"""
        hook_ctx = HookContext(operation_id='getEvents', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/events/export'
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        req_content_type, data, form = utils.serialize_request_body(request, operations.GetEventsRequestBody, "request", False, False, 'json')
        if req_content_type is not None and req_content_type not in ('multipart/form-data', 'multipart/mixed'):
            headers['content-type'] = req_content_type
        if data is None and form is None:
            raise Exception('request body is required')
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('POST', url, params=query_params, data=data, files=form, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        res = operations.GetEventsResponse(status_code=http_res.status_code, content_type=http_res.headers.get('Content-Type') or '', raw_response=http_res)
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[operations.GetEventsResponseBody])
                res.object = out
            else:
                content_type = http_res.headers.get('Content-Type')
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

        return res

    
    
    def create_model_event(self, request: operations.CreateModelEventRequestBody) -> operations.CreateModelEventResponse:
        r"""Create a new model event
        Please refer to our instrumentation guide for detailed information
        """
        hook_ctx = HookContext(operation_id='createModelEvent', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/events/model'
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        req_content_type, data, form = utils.serialize_request_body(request, operations.CreateModelEventRequestBody, "request", False, False, 'json')
        if req_content_type is not None and req_content_type not in ('multipart/form-data', 'multipart/mixed'):
            headers['content-type'] = req_content_type
        if data is None and form is None:
            raise Exception('request body is required')
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('POST', url, params=query_params, data=data, files=form, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        res = operations.CreateModelEventResponse(status_code=http_res.status_code, content_type=http_res.headers.get('Content-Type') or '', raw_response=http_res)
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[operations.CreateModelEventResponseBody])
                res.object = out
            else:
                content_type = http_res.headers.get('Content-Type')
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

        return res

    
    
    def create_event_batch(self, request: operations.CreateEventBatchRequestBody) -> operations.CreateEventBatchResponse:
        r"""Create a batch of events
        Please refer to our instrumentation guide for detailed information
        """
        hook_ctx = HookContext(operation_id='createEventBatch', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/events/batch'
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        req_content_type, data, form = utils.serialize_request_body(request, operations.CreateEventBatchRequestBody, "request", False, False, 'json')
        if req_content_type is not None and req_content_type not in ('multipart/form-data', 'multipart/mixed'):
            headers['content-type'] = req_content_type
        if data is None and form is None:
            raise Exception('request body is required')
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('POST', url, params=query_params, data=data, files=form, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['4XX','500','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        res = operations.CreateEventBatchResponse(status_code=http_res.status_code, content_type=http_res.headers.get('Content-Type') or '', raw_response=http_res)
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[operations.CreateEventBatchResponseBody])
                res.object = out
            else:
                content_type = http_res.headers.get('Content-Type')
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 500:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, errors.CreateEventBatchResponseBody)
                out.raw_response = http_res
                raise out
            else:
                content_type = http_res.headers.get('Content-Type')
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

        return res

    
    
    def create_model_event_batch(self, request: operations.CreateModelEventBatchRequestBody) -> operations.CreateModelEventBatchResponse:
        r"""Create a batch of model events
        Please refer to our instrumentation guide for detailed information
        """
        hook_ctx = HookContext(operation_id='createModelEventBatch', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/events/model/batch'
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        req_content_type, data, form = utils.serialize_request_body(request, operations.CreateModelEventBatchRequestBody, "request", False, False, 'json')
        if req_content_type is not None and req_content_type not in ('multipart/form-data', 'multipart/mixed'):
            headers['content-type'] = req_content_type
        if data is None and form is None:
            raise Exception('request body is required')
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('POST', url, params=query_params, data=data, files=form, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['4XX','500','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        res = operations.CreateModelEventBatchResponse(status_code=http_res.status_code, content_type=http_res.headers.get('Content-Type') or '', raw_response=http_res)
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[operations.CreateModelEventBatchResponseBody])
                res.object = out
            else:
                content_type = http_res.headers.get('Content-Type')
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 500:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, errors.CreateModelEventBatchResponseBody)
                out.raw_response = http_res
                raise out
            else:
                content_type = http_res.headers.get('Content-Type')
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

        return res

    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from honeyhive import utils
from typing import List, Optional, Union


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class ExperimentResultResponseDatapoints:
    passed: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('passed'), 'exclude': lambda f: f is None }})
    failed: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('failed'), 'exclude': lambda f: f is None }})
    



@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Details:
    metric_name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metric_name'), 'exclude': lambda f: f is None }})
    metric_type: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metric_type'), 'exclude': lambda f: f is None }})
    event_name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('event_name'), 'exclude': lambda f: f is None }})
    event_type: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('event_type'), 'exclude': lambda f: f is None }})
    aggregate: Optional[float] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('aggregate'), 'exclude': lambda f: f is None }})
    values: Optional[List[Values]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('values'), 'exclude': lambda f: f is None }})
    datapoints: Optional[ExperimentResultResponseDatapoints] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('datapoints'), 'exclude': lambda f: f is None }})
    



@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Metrics:
    aggregation_function: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('aggregation_function'), 'exclude': lambda f: f is None }})
    details: Optional[List[Details]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('details'), 'exclude': lambda f: f is None }})
    



@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class ExperimentResultResponseMetrics:
    name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name'), 'exclude': lambda f: f is None }})
    event_name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('event_name'), 'exclude': lambda f: f is None }})
    event_type: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('event_type'), 'exclude': lambda f: f is None }})
    value: Optional[Value] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('value'), 'exclude': lambda f: f is None }})
    passed: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('passed'), 'exclude': lambda f: f is None }})
    



@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Datapoints:
    datapoint_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('datapoint_id'), 'exclude': lambda f: f is None }})
    session_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('session_id'), 'exclude': lambda f: f is None }})
    passed: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('passed'), 'exclude': lambda f: f is None }})
    metrics: Optional[List[ExperimentResultResponseMetrics]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metrics'), 'exclude': lambda f: f is None }})
    



@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class ExperimentResultResponse:
    status: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('status'), 'exclude': lambda f: f is None }})
    success: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('success'), 'exclude': lambda f: f is None }})
    passed: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('passed'), 'exclude': lambda f: f is None }})
    failed: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('failed'), 'exclude': lambda f: f is None }})
    metrics: Optional[Metrics] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metrics'), 'exclude': lambda f: f is None }})
    datapoints: Optional[List[Datapoints]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('datapoints'), 'exclude': lambda f: f is None }})
    


Values = Union[float, bool]

Value = Union[float, bool]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .configuration import *
from .createdatapointrequest import *
from .createdatasetrequest import *
from .createeventrequest import *
from .createmodelevent import *
from .createprojectrequest import *
from .createrunrequest import *
from .createrunresponse import *
from .createtoolrequest import *
from .datapoint import *
from .dataset import *
from .datasetupdate import *
from .deleterunresponse import *
from .evaluationrun import *
from .event import *
from .eventfilter import *
from .experimentcomparisonresponse import *
from .experimentresultresponse import *
from .getrunresponse import *
from .getrunsresponse import *
from .metric import *
from .metricedit import *
from .postconfigurationrequest import *
from .project import *
from .putconfigurationrequest import *
from .security import *
from .sessionpropertiesbatch import *
from .sessionstartrequest import *
from .tool import *
from .updatedatapointrequest import *
from .updateprojectrequest import *
from .updaterunrequest import *
from .updaterunresponse import *
from .updatetoolrequest import *

__all__ = ["CallType","Configuration","ConfigurationType","CreateDatapointRequest","CreateDatasetRequest","CreateDatasetRequestPipelineType","CreateDatasetRequestType","CreateEventRequest","CreateEventRequestEventType","CreateModelEvent","CreateProjectRequest","CreateRunRequest","CreateRunResponse","CreateToolRequest","CreateToolRequestType","Datapoint","Datapoints","Dataset","DatasetType","DatasetUpdate","DeleteRunResponse","Details","Env","EvaluationRun","EvaluationRunStatus","Evaluators","Event","EventDetails","EventFilter","EventType","ExperimentComparisonResponse","ExperimentComparisonResponseConfiguration","ExperimentComparisonResponseEvaluators","ExperimentComparisonResponseMetadata","ExperimentComparisonResponseMetrics","ExperimentComparisonResponsePassingRanges","ExperimentComparisonResponseResults","ExperimentComparisonResponseSchemasConfiguration","ExperimentComparisonResponseSchemasResults","ExperimentResultResponse","ExperimentResultResponseDatapoints","ExperimentResultResponseMetrics","FunctionCallParams","GetRunResponse","GetRunsResponse","Metadata","Metric","MetricEdit","MetricEditEventType","MetricEditReturnType","MetricEditThreshold","MetricEditType","MetricType","Metrics","NewRun","NewValues","OldRun","OldValues","Operator","Parameters","PassingRanges","PipelineType","PostConfigurationRequest","PostConfigurationRequestCallType","PostConfigurationRequestEnv","PostConfigurationRequestFunctionCallParams","PostConfigurationRequestParameters","PostConfigurationRequestResponseFormat","PostConfigurationRequestSelectedFunctions","Project","PutConfigurationRequest","PutConfigurationRequestCallType","PutConfigurationRequestEnv","PutConfigurationRequestFunctionCallParams","PutConfigurationRequestParameters","PutConfigurationRequestResponseFormat","PutConfigurationRequestSelectedFunctions","PutConfigurationRequestType","ResponseFormat","Results","ReturnType","Security","SelectedFunctions","SessionPropertiesBatch","SessionStartRequest","Status","Threshold","Tool","ToolType","Type","UpdateDatapointRequest","UpdateProjectRequest","UpdateRunRequest","UpdateRunRequestStatus","UpdateRunResponse","UpdateToolRequest","Value","Values"]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from honeyhive import utils
from typing import Optional


class Operator(str, Enum):
    r"""The type of filter you are performing - \\"is\\", \\"is not\\", \\"contains\\", \\"not contains\\", \\"greater than\\" """
    IS = 'is'
    IS_NOT = 'is not'
    CONTAINS = 'contains'
    NOT_CONTAINS = 'not contains'
    GREATER_THAN = 'greater than'


class Type(str, Enum):
    r"""The data type you are using - \\"string\\", \\"number\\", \\"boolean\\", \\"id\\" (for object ids)"""
    STRING = 'string'
    NUMBER = 'number'
    BOOLEAN = 'boolean'
    ID = 'id'


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class EventFilter:
    field: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('field'), 'exclude': lambda f: f is None }})
    r"""The field name that you are filtering by like `metadata.cost`, `inputs.chat_history.0.content`"""
    value: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('value'), 'exclude': lambda f: f is None }})
    r"""The value that you are filtering the field for"""
    operator: Optional[Operator] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('operator'), 'exclude': lambda f: f is None }})
    r"""The type of filter you are performing - \\"is\\", \\"is not\\", \\"contains\\", \\"not contains\\", \\"greater than\\" """
    type: Optional[Type] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('type'), 'exclude': lambda f: f is None }})
    r"""The data type you are using - \\"string\\", \\"number\\", \\"boolean\\", \\"id\\" (for object ids)"""
    




from honeyhive.sdk import HoneyHive
from honeyhive.models import components
from honeyhive import HoneyHiveTracer, enrich_session
from .evaluators import evaluator, aevaluator

from concurrent.futures import ThreadPoolExecutor
import collections

from rich.style import Style
from rich.console import Console
from rich.table import Table

from dataclasses import dataclass
from typing import Optional, List, Dict, Any, Callable
import os
import hashlib
import json
import time
import sys
import traceback
import asyncio
import inspect

@dataclass
class EvaluationResult:
    run_id: str
    stats: Dict[str, Any]
    dataset_id: str 
    session_ids: list
    status: str
    suite: str
    data: Dict[str, list]

    def to_json(self):
        # save data dict to json file
        with open(f"{self.suite}.json", "w") as f:
            json.dump(self.data, f, indent=4)

class DatasetLoader:

    @staticmethod
    def load_dataset(hhai: HoneyHive, project: str, dataset_id: str) -> Optional[Dict[str, Any]]:
        """Private function to acquire Honeyhive dataset based on dataset_id."""
        if not dataset_id:
            return None
        try:
            dataset = hhai.datasets.get_datasets(
                project=project,
                dataset_id=dataset_id,
            )
            if (
                dataset
                and dataset.object.testcases
                and len(dataset.object.testcases) > 0
            ):
                return dataset.object.testcases[0]
        except Exception:
            raise RuntimeError(
                f"No dataset found with id - {dataset_id} for project - {project}"
            )


console = Console()

class Evaluation:
    """This class is for automated honeyhive evaluation with tracing"""

    def __init__(
        self,
        hh_api_key: str = None,
        hh_project: str = None,
        name: Optional[str] = None,
        suite: Optional[str] = None,
        function: Optional[Callable] = None,
        dataset: Optional[List[Any]] = None,
        evaluators: Optional[List[Any]] = None,
        dataset_id: Optional[str] = None,
        max_workers: int = 10,
        run_concurrently: bool = True,
        server_url: Optional[str] = None,
        verbose: bool = False,
    ):
        
        if function is None:
            raise Exception(
                "Please provide a function to evaluate."
            )
        
        # if name is not provided, use the file name
        try:
            if name is None:
                name = os.path.basename(sys._getframe(1).f_code.co_filename)
        except Exception:
            name = "default"

        # get the directory of the file being evaluated
        try:
            if suite is None:
                suite = os.path.dirname(sys._getframe(1).f_code.co_filename).split(os.sep)[-1]
        except Exception:
            suite = "default"
        
        self.hh_api_key = hh_api_key or os.environ["HH_API_KEY"]
        self.hh_project = hh_project or os.environ["HH_PROJECT"]
        self.eval_name: str = name
        self.hh_dataset_id: Optional[str] = dataset_id
        self.client_side_evaluators = evaluators or []
        self.status: str = "pending"
        self.max_workers: int = max_workers
        self.run_concurrently: bool = run_concurrently
        self.dataset = dataset
        self.func_to_evaluate: Callable = function
        self.suite = suite
        self.disable_auto_tracing = True
        self.eval_run: Optional[components.CreateRunResponse] = None
        self.evaluation_session_ids: collections.deque = collections.deque()
        self._validate_requirements()

        self.hhai = HoneyHive(bearer_auth=self.hh_api_key)
        self.hh_dataset = DatasetLoader.load_dataset(self.hhai, self.hh_project, self.hh_dataset_id)

        self.server_url = server_url
        self.verbose = verbose

        # generated id for external datasets
        # TODO: large dataset optimization
        # TODO: dataset might not be json serializable
        self.external_dataset_id: str = (
            Evaluation.generate_hash(json.dumps(dataset)) if dataset else None
        )

        # increase the OTEL export timeout to 30 seconds
        os.environ["OTEL_EXPORTER_OTLP_TIMEOUT"] = "30000"

    def _validate_requirements(self) -> None:
        """Sanity check of requirements for HoneyHive evaluations and tracing."""
        if not self.hh_api_key:
            raise Exception(
                "Honeyhive API key not found. Please set 'hh_api_key' to initiate Honeyhive Tracer. Cannot run Evaluation"
            )
        if not self.hh_project:
            raise Exception(
                "Honeyhive Project not found. Please set 'hh_project' to initiate Honeyhive Tracer. Cannot run Evaluation"
            )
        if not self.eval_name:
            raise Exception(
                "Evaluation name not found. Please set 'name' to initiate Honeyhive Evaluation."
            )
        if not self.hh_dataset_id and not self.dataset:
            raise Exception(
                "No valid 'dataset_id' or 'dataset' found. Please provide one to iterate the evaluation over."
            )
        if self.dataset is not None:
            if not isinstance(self.dataset, list):
                raise Exception("Dataset must be a list")
            if not all(isinstance(item, dict) for item in self.dataset):
                raise Exception("All items in dataset must be dictionaries")

    @staticmethod
    def generate_hash(input_string: str) -> str:
        return f"EXT-{hashlib.md5(input_string.encode('utf-8')).hexdigest()[:24]}"

    # ------------------------------------------------------------

    def _enrich_evaluation_session(
        self,
        datapoint_idx: int,
        session_id: str,
        outputs: Optional[Any],
        metrics: Optional[Dict[str, Any]] = None,
        metadata: Optional[Dict[str, Any]] = None,
    ):
        """Private function to enrich the session data post flow completion."""
        try:
            tracing_metadata = {"run_id": self.eval_run.run_id}
            if self.hh_dataset:
                tracing_metadata["datapoint_id"] = self.hh_dataset.datapoints[datapoint_idx]
                tracing_metadata["dataset_id"] = self.hh_dataset_id
            if self.external_dataset_id:
                tracing_metadata["datapoint_id"] = Evaluation.generate_hash(
                    json.dumps(self.dataset[datapoint_idx])
                )
                tracing_metadata["dataset_id"] = self.external_dataset_id

            if not isinstance(outputs, dict):
                outputs = {"output": outputs}

            tracing_metadata.update(metadata)

            enrich_session(
                session_id=session_id,
                metadata=tracing_metadata,
                outputs=outputs,
                metrics=metrics,
            )
        except Exception as e:
            print(f"Error adding trace metadata: {e}")
    
    def _enrich_sessions(
        self,
        results: List[Dict[str, Any]],
    ):
        """Private function to enrich the session data post flow completion."""
        try:
            for idx, result in enumerate(results):
                tracing_metadata = {"run_id": self.eval_run.run_id}
                if self.hh_dataset:
                    tracing_metadata["datapoint_id"] = self.hh_dataset.datapoints[idx]
                tracing_metadata["dataset_id"] = self.hh_dataset_id

                if self.external_dataset_id:
                    tracing_metadata["datapoint_id"] = Evaluation.generate_hash(
                        json.dumps(self.dataset[idx])
                    )
                    tracing_metadata["dataset_id"] = self.external_dataset_id

                if not isinstance(result['output'], dict):
                    result['output'] = {"output": result['output']}

                tracing_metadata.update(result['metadata'])

                enrich_session(
                    session_id=self.evaluation_session_ids[idx],
                    metadata=tracing_metadata,
                    outputs=result['output'],
                    metrics=result['metrics'],
                )
        except Exception as e:
            print(f"Error adding trace metadata: {e}")

    def _get_evaluator_metadata(self, eval_func, evaluator_name: str) -> dict:
        """Get metadata for an evaluator if it's decorated."""
        if not isinstance(eval_func, evaluator):
            return {}
            
        eval_settings_dict = evaluator.all_evaluator_settings[evaluator_name].resolve_settings().dict()
        # remove all None values, weight if 1.0 and asserts if False
        filtered_dict = {}
        for k, v in eval_settings_dict.items():
            if not (v is None or (k == 'weight' and v == 1.0) or (k == 'asserts' and v is False)):
                filtered_dict[k] = v
        return {
            'eval_settings': filtered_dict,
        }

    def _run_single_evaluator(
        self,
        eval_func,
        evaluator_name: str,
        outputs: Any,
        inputs: Dict[str, Any],
        ground_truth: Dict[str, Any]
    ) -> tuple[str, Any, dict]:
        """Run a single evaluator and return its name, result and metadata."""
        metadata = self._get_evaluator_metadata(eval_func, evaluator_name)

        try:
            # if evaluator takes 1 argument, pass outputs
            if eval_func.__code__.co_argcount == 1:
                evaluator_result = eval_func(outputs)
            # if evaluator takes 2 arguments, pass outputs and inputs
            elif eval_func.__code__.co_argcount == 2:
                evaluator_result = eval_func(outputs, inputs)
            # if evaluator takes 3 arguments, pass outputs, inputs, and ground_truth
            elif eval_func.__code__.co_argcount == 3:
                evaluator_result = eval_func(outputs, inputs, ground_truth)
            else:
                raise ValueError(f"Evaluator {evaluator_name} must accept either 1, 2, or 3 arguments (outputs, inputs, ground_truth)")
            
            return evaluator_name, evaluator_result, metadata

        except AssertionError:
            return evaluator_name, None, metadata
        except Exception as e:
            print(f"Error in evaluator: {str(e)}\n")
            print(traceback.format_exc())
            return evaluator_name, None, metadata

    async def _arun_single_evaluator(
        self,
        eval_func,
        evaluator_name: str,
        outputs: Any,
        inputs: Dict[str, Any],
        ground_truth: Dict[str, Any]
    ) -> tuple[str, Any, dict]:
        """Run a single async evaluator and return its name, result and metadata."""
        metadata = self._get_evaluator_metadata(eval_func, evaluator_name)

        try:
            # if evaluator takes 1 argument, pass outputs
            if eval_func.__code__.co_argcount == 1:
                evaluator_result = await eval_func(outputs)
            # if evaluator takes 2 arguments, pass outputs and inputs
            elif eval_func.__code__.co_argcount == 2:
                evaluator_result = await eval_func(outputs, inputs)
            # if evaluator takes 3 arguments, pass outputs, inputs, and ground_truth
            elif eval_func.__code__.co_argcount == 3:
                evaluator_result = await eval_func(outputs, inputs, ground_truth)
            else:
                raise ValueError(f"Evaluator {evaluator_name} must accept either 1, 2, or 3 arguments (outputs, inputs, ground_truth)")
            
            return evaluator_name, evaluator_result, metadata

        except AssertionError:
            return evaluator_name, None, metadata
        except Exception as e:
            print(f"Error in evaluator: {str(e)}\n")
            print(traceback.format_exc())
            return evaluator_name, None, metadata

    def _run_evaluators(
        self, 
        outputs: Optional[Any], 
        inputs: Optional[Dict[str, Any]], 
        ground_truth: Optional[Dict[str, Any]]
    ):
        """Run evaluators and collect metrics."""
        metrics = {}
        metadata = {}

        if not self.client_side_evaluators:
            return metrics, metadata

        # Separate sync and async evaluators
        sync_evaluators = []
        async_evaluators = []
        eval_names = set()

        for index, eval_func in enumerate(self.client_side_evaluators):
            evaluator_name = getattr(eval_func, "__name__", f"evaluator_{index}")
            if evaluator_name in eval_names:
                raise ValueError(f"Evaluator {evaluator_name} is defined multiple times")
            eval_names.add(evaluator_name)

            if inspect.iscoroutinefunction(eval_func):
                async_evaluators.append((eval_func, evaluator_name))
            else:
                sync_evaluators.append((eval_func, evaluator_name))

        # Run sync evaluators first
        for eval_func, name in sync_evaluators:
            name, result, meta = self._run_single_evaluator(
                eval_func, name, outputs, inputs, ground_truth
            )
            metrics[name] = result
            if meta:
                metadata[name] = meta

        # Run async evaluators concurrently if any exist
        if async_evaluators:
            print('Evaluators cannot be run async. Please use sync evaluators only.')
        # if async_evaluators:

        #     async def arun_async_evaluators():
        #         async_tasks = [
        #             self._arun_single_evaluator(eval_func, name, outputs, inputs, ground_truth)
        #             for eval_func, name in async_evaluators
        #         ]
        #         return await asyncio.gather(*async_tasks)

        #     async_results = asyncio.run(arun_async_evaluators())

        #     for name, result, meta in async_results:
        #         metrics[name] = result
        #         if meta:
        #             metadata[name] = meta

        return metrics, metadata

    def _create_result(self, inputs, ground_truth, outputs, metrics, metadata):
        """Create standardized result dictionary."""
        return {
            'input': inputs,
            'ground_truth': ground_truth,
            'output': outputs,
            'metrics': metrics,
            'metadata': metadata,
        }

    def _get_inputs_and_ground_truth(self, datapoint_idx: int):
        """Get inputs and ground truth for evaluation from dataset."""
        if (
            self.hh_dataset
            and self.hh_dataset.datapoints
            and len(self.hh_dataset.datapoints) > 0
        ):
            datapoint_id = self.hh_dataset.datapoints[datapoint_idx]
            datapoint_response = self.hhai.datapoints.get_datapoint(id=datapoint_id)
            return (
                datapoint_response.object.datapoint[0].inputs or {}, 
                datapoint_response.object.datapoint[0].ground_truth or {}
            )
        elif self.dataset:
            return (
                self.dataset[datapoint_idx].get('inputs', {}), 
                self.dataset[datapoint_idx].get('ground_truths', {})
            )
        return ({}, {})

    def _init_tracer(self, inputs: Dict[str, Any]) -> HoneyHiveTracer:
        """Initialize HoneyHiveTracer for evaluation."""
        hh = HoneyHiveTracer(
            api_key=self.hh_api_key,
            project=self.hh_project,
            source="evaluation",
            session_name=self.eval_name,
            inputs={'inputs': inputs},
            is_evaluation=True,
            verbose=self.verbose,
            server_url=self.server_url,
        )
        return hh

    def run_each(self, datapoint_idx: int) -> Dict[str, Any]:
        """Run evaluation for a single datapoint in its own thread."""

        inputs = {}
        ground_truth = {}
        outputs = None
        metrics = {}
        metadata = {}
        session_id = None

        # Get inputs
        try:
            inputs, ground_truth = self._get_inputs_and_ground_truth(datapoint_idx)
        except Exception as e:
            print(f"Error getting inputs for index {datapoint_idx}: {e}")
            return self._create_result(inputs, ground_truth, outputs, metrics, metadata)

        # Initialize tracer
        try:
            hh = self._init_tracer(inputs)
            session_id = hh.session_id
            self.evaluation_session_ids.append(session_id)
        except Exception as e:
            print(traceback.format_exc())
            raise Exception(
                f"Unable to initiate Honeyhive Tracer. Cannot run Evaluation: {e}"
            )
        # Run the function
        try:
            if inspect.iscoroutinefunction(self.func_to_evaluate):
                raise ValueError("Evaluation task must be sync. Please use asyncio.run() to run coroutines inside the task.")
            else:
                if self.func_to_evaluate.__code__.co_argcount == 2:
                    outputs = self.func_to_evaluate(inputs, ground_truth)
                elif self.func_to_evaluate.__code__.co_argcount == 1:
                    outputs = self.func_to_evaluate(inputs)
                else:
                    raise ValueError(f"Evaluation function must accept either 1 or 2 arguments (inputs, ground_truth)")
        except Exception as e:
            print(f"Error in evaluation function: {e}")
            print(traceback.format_exc())
        
        # Run evaluators
        metrics, metadata = self._run_evaluators(outputs, inputs, ground_truth)
        
        # Add trace metadata, outputs, and metrics to session

        self._enrich_evaluation_session(
            datapoint_idx,
            session_id, 
            outputs,
            metrics,
            metadata
        )

        console.print(f"Test case {datapoint_idx} complete")
        
        return self._create_result(inputs, ground_truth, outputs, metrics, metadata)

    def run(self):
        """Public function to run the evaluation."""

        # create run
        eval_run = self.hhai.experiments.create_run(
            request=components.CreateRunRequest(
                project=self.hh_project,
                name=self.eval_name,
                dataset_id=self.hh_dataset_id or self.external_dataset_id,
                event_ids=[],
                status=self.status,
            )
        )
        self.eval_run = eval_run.create_run_response

        self.eval_result = EvaluationResult(
            run_id=self.eval_run.run_id,
            dataset_id=self.hh_dataset_id or self.external_dataset_id,
            session_ids=[],
            status=self.status,
            suite=self.suite,
            stats={},
            data={},
        )

        #########################################################
        # Run evaluations
        #########################################################

        if self.hh_dataset:
            num_points = len(self.hh_dataset.datapoints)
        elif self.dataset:
            num_points = len(self.dataset)
        else:
            raise Exception("No dataset found")
        
        start_time = time.time()
        # Use ThreadPoolExecutor to run evaluations concurrently
        max_workers = os.getenv("HH_MAX_WORKERS", 10)
        with console.status("[bold green]Working on evals...") as status:
            with ThreadPoolExecutor(max_workers=max_workers) as executor:
                try:
                    # Submit tasks and get futures
                    futures = [executor.submit(self.run_each, i) for i in range(num_points)]
                    
                    # Collect results and log any errors
                    results = []
                    for future in futures:
                        try:
                            results.append(future.result())
                        except Exception as e:
                            print(f"Error in evaluation thread: {e}")
                            # Still add None result to maintain ordering
                            results.append(None)
                except KeyboardInterrupt:
                    executor.shutdown(wait=False)
                    raise
        end_time = time.time()
        #########################################################

        # enrich the sessions
        # Add trace metadata, outputs, and metrics
        # try:
        #     self._enrich_sessions(results)
        # except Exception as e:
        #     print(f"Error adding trace metadata: {e}")

        # Process results
        self.eval_result.stats = {
            'duration_s': round(end_time - start_time, 3),
        }
        self.eval_result.data = {
            'input': [],
            'output': [],
            'metrics': [],
            'metadata': [],
            'ground_truth': []
        }
        for r in results:
            for k in self.eval_result.data.keys():
                self.eval_result.data[k].append(r[k])

        # Convert deque to list after all threads complete
        self.eval_result.session_ids = list(self.evaluation_session_ids)

        #########################################################
        # Update run
        #########################################################
        try:
            if self.eval_run:
                self.status = "completed"
                self.hhai.experiments.update_run(
                    run_id=self.eval_run.run_id,
                    update_run_request=components.UpdateRunRequest(
                        event_ids=self.eval_result.session_ids, 
                        status=self.status
                    ),
                )
        except Exception:
            print("Warning: Unable to mark evaluation as `Completed`")


    def print_run(self):
        """Print the results of the evaluation."""

        # get column names
        input_cols = {k for result in self.eval_result.data['input'] for k in result.keys()}
        metric_cols = {k for result in self.eval_result.data['metrics'] for k in result.keys()}
        metadata_cols = {k for result in self.eval_result.data['metadata'] for k in result.keys()}
        ground_truth_cols = {k for result in self.eval_result.data['ground_truth'] for k in result.keys()}

        # make table
        table = Table(
            title=f"Evaluation Results: {self.eval_name}",
            show_lines=True,
            title_style=Style(
                color="black",
                bgcolor="yellow",
                bold=True,
                frame=True,
            ),
        )
        table.add_column("Suite", justify="center", style="magenta")
        for k in input_cols:
            table.add_column(f'Inputs.{k}', justify="center", style="green")
        table.add_column("Outputs", justify="center", style="blue")
        for k in ground_truth_cols:
            table.add_column(f'Ground Truths.{k}', justify="center", style="green")
        for k in metric_cols:
            table.add_column(f'Metrics.{k}', justify="center", style="blue")
        for k in metadata_cols:
            table.add_column(f'Metadata.{k}', justify="center", style="green")

        def truncated(string, max_length=500):
            if len(string) > max_length:
                return string[:max_length] + "..."
            return string

        # Get length of any list in data dict since they're all equal length
        n_rows = len(self.eval_result.data['input'])
        
        for idx in range(n_rows):
            row_values = [self.eval_result.suite]
            # Add input columns
            for k in input_cols:
                row_values.append(truncated(str(self.eval_result.data['input'][idx].get(k, ''))))
            # Add output column
            row_values.append(truncated(str(self.eval_result.data['output'][idx])))
            # Add ground truth columns
            for k in ground_truth_cols:
                row_values.append(truncated(str(self.eval_result.data['ground_truth'][idx].get(k, ''))))
            # Add metric columns
            for k in metric_cols:
                row_values.append(truncated(str(self.eval_result.data['metrics'][idx].get(k, ''))))
            # Add metadata columns
            for k in metadata_cols:
                row_values.append(truncated(str(self.eval_result.data['metadata'][idx].get(k, ''))))
            table.add_row(*row_values)

        console.print(table)

        # add footer with evaluation duration
        print(f"Evaluation Duration: {self.eval_result.stats['duration_s']} seconds\n")

        print('Exporting traces to HoneyHive...')


def evaluate(*args, **kwargs):

    eval = Evaluation(*args, **kwargs)

    # run evaluation
    eval.run()

    # print evaluation results
    eval.print_run()

    return EvaluationResult(
        run_id=eval.eval_run.run_id,
        dataset_id=eval.hh_dataset_id or eval.external_dataset_id,
        session_ids=eval.evaluation_session_ids,
        status=eval.status,
        data=eval.eval_result.data,
        stats=eval.eval_result.stats,
        suite=eval.suite
    )


__all__ = [
    "evaluate",
    "evaluator",
    "aevaluator",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from honeyhive import utils
from typing import Any, Dict, List, Optional


class MetricEditType(str, Enum):
    r"""Type of the metric - \\"custom\\", \\"model\\", \\"human\\" or \\"composite\\" """
    CUSTOM = 'custom'
    MODEL = 'model'
    HUMAN = 'human'
    COMPOSITE = 'composite'


class MetricEditReturnType(str, Enum):
    r"""The data type of the metric value - \\"boolean\\", \\"float\\", \\"string\\" """
    BOOLEAN = 'boolean'
    FLOAT = 'float'
    STRING = 'string'


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class MetricEditThreshold:
    r"""Threshold for numeric metrics to decide passing or failing in tests"""
    min: Optional[float] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('min'), 'exclude': lambda f: f is None }})
    max: Optional[float] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('max'), 'exclude': lambda f: f is None }})
    



class MetricEditEventType(str, Enum):
    r"""Type of event that the metric is set to be computed on"""
    MODEL = 'model'
    TOOL = 'tool'
    CHAIN = 'chain'
    SESSION = 'session'


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class MetricEdit:
    metric_id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metric_id') }})
    r"""Unique identifier of the metric"""
    criteria: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('criteria'), 'exclude': lambda f: f is None }})
    r"""Criteria for human or composite metrics"""
    name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name'), 'exclude': lambda f: f is None }})
    r"""Updated name of the metric"""
    description: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('description'), 'exclude': lambda f: f is None }})
    r"""Short description of what the metric does"""
    code_snippet: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('code_snippet'), 'exclude': lambda f: f is None }})
    r"""Updated code block for the metric"""
    prompt: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('prompt'), 'exclude': lambda f: f is None }})
    r"""Updated Evaluator prompt for the metric"""
    type: Optional[MetricEditType] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('type'), 'exclude': lambda f: f is None }})
    r"""Type of the metric - \\"custom\\", \\"model\\", \\"human\\" or \\"composite\\" """
    enabled_in_prod: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('enabled_in_prod'), 'exclude': lambda f: f is None }})
    r"""Whether to compute on all production events automatically"""
    needs_ground_truth: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('needs_ground_truth'), 'exclude': lambda f: f is None }})
    r"""Whether a ground truth (on metadata) is required to compute it"""
    return_type: Optional[MetricEditReturnType] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('return_type'), 'exclude': lambda f: f is None }})
    r"""The data type of the metric value - \\"boolean\\", \\"float\\", \\"string\\" """
    threshold: Optional[MetricEditThreshold] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('threshold'), 'exclude': lambda f: f is None }})
    r"""Threshold for numeric metrics to decide passing or failing in tests"""
    pass_when: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('pass_when'), 'exclude': lambda f: f is None }})
    r"""Threshold for boolean metrics to decide passing or failing in tests"""
    event_name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('event_name'), 'exclude': lambda f: f is None }})
    r"""Name of event that the metric is set to be computed on"""
    event_type: Optional[MetricEditEventType] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('event_type'), 'exclude': lambda f: f is None }})
    r"""Type of event that the metric is set to be computed on"""
    child_metrics: Optional[List[Dict[str, Any]]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('child_metrics'), 'exclude': lambda f: f is None }})
    r"""Child metrics added under composite events"""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from honeyhive import utils
from typing import Any, Dict, List, Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Datapoint:
    id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('_id'), 'exclude': lambda f: f is None }})
    r"""UUID for the datapoint"""
    tenant: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('tenant'), 'exclude': lambda f: f is None }})
    project_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('project_id'), 'exclude': lambda f: f is None }})
    r"""UUID for the project where the datapoint is stored"""
    created_at: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('created_at'), 'exclude': lambda f: f is None }})
    updated_at: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('updated_at'), 'exclude': lambda f: f is None }})
    inputs: Optional[Dict[str, Any]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('inputs'), 'exclude': lambda f: f is None }})
    r"""Arbitrary JSON object containing the inputs for the datapoint"""
    history: Optional[List[Dict[str, Any]]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('history'), 'exclude': lambda f: f is None }})
    r"""Conversation history associated with the datapoint"""
    ground_truth: Optional[Dict[str, Any]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('ground_truth'), 'exclude': lambda f: f is None }})
    linked_event: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('linked_event'), 'exclude': lambda f: f is None }})
    r"""Event id for the event from which the datapoint was created"""
    linked_evals: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('linked_evals'), 'exclude': lambda f: f is None }})
    r"""Ids of evaluations where the datapoint is included"""
    linked_datasets: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('linked_datasets'), 'exclude': lambda f: f is None }})
    r"""Ids of all datasets that include the datapoint"""
    saved: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('saved'), 'exclude': lambda f: f is None }})
    type: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('type'), 'exclude': lambda f: f is None }})
    r"""session or event - specify the type of data"""
    metadata: Optional[Dict[str, Any]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata'), 'exclude': lambda f: f is None }})
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from honeyhive import utils
from typing import List, Optional, Union


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class ExperimentComparisonResponseMetrics:
    metric_name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metric_name'), 'exclude': lambda f: f is None }})
    event_name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('event_name'), 'exclude': lambda f: f is None }})
    metric_type: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metric_type'), 'exclude': lambda f: f is None }})
    event_type: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('event_type'), 'exclude': lambda f: f is None }})
    old_aggregate: Optional[float] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('old_aggregate'), 'exclude': lambda f: f is None }})
    new_aggregate: Optional[float] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('new_aggregate'), 'exclude': lambda f: f is None }})
    found_count: Optional[int] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('found_count'), 'exclude': lambda f: f is None }})
    improved_count: Optional[int] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('improved_count'), 'exclude': lambda f: f is None }})
    degraded_count: Optional[int] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('degraded_count'), 'exclude': lambda f: f is None }})
    same_count: Optional[int] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('same_count'), 'exclude': lambda f: f is None }})
    improved: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('improved'), 'exclude': lambda f: f is None }})
    degraded: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('degraded'), 'exclude': lambda f: f is None }})
    same: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('same'), 'exclude': lambda f: f is None }})
    old_values: Optional[List[OldValues]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('old_values'), 'exclude': lambda f: f is None }})
    new_values: Optional[List[NewValues]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('new_values'), 'exclude': lambda f: f is None }})
    



@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class EventDetails:
    event_name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('event_name'), 'exclude': lambda f: f is None }})
    event_type: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('event_type'), 'exclude': lambda f: f is None }})
    presence: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('presence'), 'exclude': lambda f: f is None }})
    



@dataclasses.dataclass
class Evaluators:
    pass


@dataclasses.dataclass
class ExperimentComparisonResponseResults:
    pass


@dataclasses.dataclass
class ExperimentComparisonResponseSchemasConfiguration:
    pass


@dataclasses.dataclass
class Metadata:
    pass


@dataclasses.dataclass
class PassingRanges:
    pass


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class OldRun:
    id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('_id'), 'exclude': lambda f: f is None }})
    run_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('run_id'), 'exclude': lambda f: f is None }})
    project: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('project'), 'exclude': lambda f: f is None }})
    tenant: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('tenant'), 'exclude': lambda f: f is None }})
    created_at: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('created_at'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    event_ids: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('event_ids'), 'exclude': lambda f: f is None }})
    session_ids: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('session_ids'), 'exclude': lambda f: f is None }})
    dataset_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('dataset_id'), 'exclude': lambda f: f is None }})
    datapoint_ids: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('datapoint_ids'), 'exclude': lambda f: f is None }})
    evaluators: Optional[List[Evaluators]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('evaluators'), 'exclude': lambda f: f is None }})
    results: Optional[ExperimentComparisonResponseResults] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('results'), 'exclude': lambda f: f is None }})
    configuration: Optional[ExperimentComparisonResponseSchemasConfiguration] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('configuration'), 'exclude': lambda f: f is None }})
    metadata: Optional[Metadata] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata'), 'exclude': lambda f: f is None }})
    passing_ranges: Optional[PassingRanges] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('passing_ranges'), 'exclude': lambda f: f is None }})
    status: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('status'), 'exclude': lambda f: f is None }})
    name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name'), 'exclude': lambda f: f is None }})
    



@dataclasses.dataclass
class ExperimentComparisonResponseEvaluators:
    pass


@dataclasses.dataclass
class ExperimentComparisonResponseSchemasResults:
    pass


@dataclasses.dataclass
class ExperimentComparisonResponseConfiguration:
    pass


@dataclasses.dataclass
class ExperimentComparisonResponseMetadata:
    pass


@dataclasses.dataclass
class ExperimentComparisonResponsePassingRanges:
    pass


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class NewRun:
    id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('_id'), 'exclude': lambda f: f is None }})
    run_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('run_id'), 'exclude': lambda f: f is None }})
    project: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('project'), 'exclude': lambda f: f is None }})
    tenant: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('tenant'), 'exclude': lambda f: f is None }})
    created_at: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('created_at'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    event_ids: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('event_ids'), 'exclude': lambda f: f is None }})
    session_ids: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('session_ids'), 'exclude': lambda f: f is None }})
    dataset_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('dataset_id'), 'exclude': lambda f: f is None }})
    datapoint_ids: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('datapoint_ids'), 'exclude': lambda f: f is None }})
    evaluators: Optional[List[ExperimentComparisonResponseEvaluators]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('evaluators'), 'exclude': lambda f: f is None }})
    results: Optional[ExperimentComparisonResponseSchemasResults] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('results'), 'exclude': lambda f: f is None }})
    configuration: Optional[ExperimentComparisonResponseConfiguration] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('configuration'), 'exclude': lambda f: f is None }})
    metadata: Optional[ExperimentComparisonResponseMetadata] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata'), 'exclude': lambda f: f is None }})
    passing_ranges: Optional[ExperimentComparisonResponsePassingRanges] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('passing_ranges'), 'exclude': lambda f: f is None }})
    status: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('status'), 'exclude': lambda f: f is None }})
    name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name'), 'exclude': lambda f: f is None }})
    



@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class ExperimentComparisonResponse:
    metrics: Optional[List[ExperimentComparisonResponseMetrics]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metrics'), 'exclude': lambda f: f is None }})
    common_datapoints: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('commonDatapoints'), 'exclude': lambda f: f is None }})
    event_details: Optional[List[EventDetails]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('event_details'), 'exclude': lambda f: f is None }})
    old_run: Optional[OldRun] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('old_run'), 'exclude': lambda f: f is None }})
    new_run: Optional[NewRun] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('new_run'), 'exclude': lambda f: f is None }})
    


OldValues = Union[float, bool]

NewValues = Union[float, bool]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .evaluationrun import EvaluationRun
from dataclasses_json import Undefined, dataclass_json
from honeyhive import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class GetRunResponse:
    evaluation: Optional[EvaluationRun] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('evaluation'), 'exclude': lambda f: f is None }})
    




#!/usr/bin/env python

"""
distutils/setuptools install script.
"""

from setuptools import setup, find_packages

package_data = {'': ['*']}

requires = [
    'requests>=2.25.1',
    'pydantic>=1.8.2',
    'uplink>=0.0.2',
    'traceloop-sdk>=0.18.2,<=0.30',
    'typing-inspect>=0.9.0',
    'dataclasses-json>=0.6.7',
    'pyyaml>=6.0.1',
    'rich>=13.9.4',
    'openai>=1.57.2'
]

entry_points = {'console_scripts': ['honeyhive = honeyhive.cli.__main__:main']}

setup(
    name='honeyhive',
    version='0.2.32',
    description='The HoneyHive SDK for Python',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='HoneyHive',
    author_email="support@honeyhive.ai",
    scripts=[],
    packages=find_packages(where='src'),
    package_dir={'': 'src'},
    package_data=package_data,
    include_package_data=True,
    install_requires=requires,
    license="Apache License 2.0",
    python_requires=">= 3.7",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
    ],
    extras_require={
        "dev":["pylint==2.16.2"],
        "langchain":["langchain>=0.0.251"],
        "llama_index":["llama_index>=0.8.13"],
        "all": ["langchain>=0.0.251", "llama_index>=0.8.13"]
    },
    project_urls={
        'Documentation': 'https://docs.honeyhive.ai/',
    },
    entry_points=entry_points,
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from honeyhive import utils
from typing import Any, Dict, Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class UpdateToolRequest:
    id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('id') }})
    name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name') }})
    parameters: Dict[str, Any] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('parameters') }})
    description: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('description'), 'exclude': lambda f: f is None }})
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import requests as requests_http
from ...models.components import experimentresultresponse as components_experimentresultresponse
from enum import Enum
from typing import Optional


class AggregateFunction(str, Enum):
    AVERAGE = 'average'
    MIN = 'min'
    MAX = 'max'
    MEDIAN = 'median'
    P95 = 'p95'
    P99 = 'p99'
    P90 = 'p90'
    SUM = 'sum'
    COUNT = 'count'


@dataclasses.dataclass
class GetExperimentResultRequest:
    run_id: str = dataclasses.field(metadata={'path_param': { 'field_name': 'run_id', 'style': 'simple', 'explode': False }})
    project_id: str = dataclasses.field(metadata={'query_param': { 'field_name': 'project_id', 'style': 'form', 'explode': True }})
    aggregate_function: Optional[AggregateFunction] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'aggregate_function', 'style': 'form', 'explode': True }})
    



@dataclasses.dataclass
class GetExperimentResultResponse:
    content_type: str = dataclasses.field()
    r"""HTTP response content type for this operation"""
    status_code: int = dataclasses.field()
    r"""HTTP response status code for this operation"""
    raw_response: requests_http.Response = dataclasses.field()
    r"""Raw HTTP response; suitable for custom response parsing"""
    experiment_result_response: Optional[components_experimentresultresponse.ExperimentResultResponse] = dataclasses.field(default=None)
    r"""Experiment result retrieved successfully"""
    




# pylint: skip-file
"""A Tracer implementation that logs runs to HoneyHive."""
from __future__ import annotations

import copy
import logging
from abc import ABC
import json
import os
import re

from enum import Enum
import inspect
from pydantic import BaseModel, ConfigDict, Field
from typing import Any, Dict, Optional, Union, List, Tuple, Callable
from datetime import timedelta
import uuid
import requests
import random

from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry

try:
    from langchain.callbacks.tracers.base import BaseTracer, TracerException
    from langchain.callbacks.tracers.schemas import (
        TracerSession,
        Run,
    )
    from langchain.input import get_colored_text
except ImportError:
    raise ImportError("Please install our langchain tracer. You can install it with `pip install honeyhive[langchain]`")

import traceback

HONEYHIVE_APP_URL = "https://api.honeyhive.ai"
LLM_CHAIN_TYPE = "llm_chain"
AGENT_CHAIN_TYPE = "agent_executor_chain"
ROLE_MAPPING = {"ai": "assistant", "human": "user", "system": "system"}


class HoneyHiveLangChainTracer(BaseTracer, ABC):
    """An implementation of BaseTracer that logs events in a session to the HoneyHive API."""

    _headers: Dict[str, Any] = {"Content-Type": "application/json"}
    _base_url: str = "https://api.honeyhive.ai"
    _env_api_key = os.getenv("HH_API_KEY")

    def __init__(
        self,
        project: str,
        name: Optional[str] = None,
        source: Optional[str] = None,
        user_properties: Optional[Dict[str, Any]] = None,
        api_key: Optional[str] = None,
        metadata: Optional[Dict[str, Any]] = None,
        verbose: bool = False,
        base_url: Optional[str] = None,
    ):
        """Initialize the HoneyHive tracer."""
        super().__init__()
        self.verbose = verbose
        if self._env_api_key:
            api_key = self._env_api_key
        elif not api_key:
            raise ValueError(
                "HoneyHive API key is not set! Please set the HH_API_KEY environment variable or pass in the api_key value."
            )

        if base_url:
            self._base_url = base_url

        if api_key:
            self._headers["Authorization"] = f"Bearer {api_key}"

        self.project = project
        self.source = source if source is not None else "langchain"
        self.name = name
        self.user_properties = user_properties
        self.metadata = metadata
        self.eval_info = None
        self.last_event_id = None
        self.last_event_metrics = None
        self.last_event_metadata = None
        if self.source == "evaluation":
            try:
                if self.metadata and "run_id" in self.metadata:
                    self.eval_info = {"run_id": self.metadata["run_id"]}
                    if "datapoint_id" in self.metadata:
                        self.eval_info["datapoint_id"] = self.metadata["datapoint_id"]
                elif self.metadata and "dataset_name" in self.metadata:
                    project_res = requests_retry_session().get(
                        url=f"{self._base_url}/projects",
                        headers=self._headers,
                        params={"name": self.project},
                    )
                    if project_res.status_code == 200:
                        project_id = project_res.json()[0]["_id"]
                        dataset_res = requests_retry_session().get(
                            url=f"{self._base_url}/datasets",
                            headers=self._headers,
                            params={
                                "name": self.metadata["dataset_name"],
                                "project": project_id,
                            },
                        )
                        if dataset_res.status_code == 200:
                            dataset = dataset_res.json()["testcases"][0]
                            dataset_id = dataset["_id"]
                            datapoint_ids = dataset["datapoints"]
                            if "run_name" in self.metadata:
                                run_name = self.metadata["run_name"]
                            else:
                                run_name = self.name
                            self.eval_info = {
                                "dataset_id": dataset_id,
                                "datapoint_ids": datapoint_ids,
                                "project_id": project_id,
                                "run_name": run_name,
                            }
            except Exception as error:
                logging.warning(f"Failed to retrieve datapoint ids: {error}")
                if self.verbose:
                    traceback.print_exc()
        if api_key is not None:
            self._headers["Authorization"] = "Bearer " + api_key

    def set_metric(
        self,
        metric_name,
        metric_value,
        threshold,
    ):
        if not self.last_event_id:
            raise Exception("No events defined on session to set metric on")
        metrics = self.last_event_metrics.copy()
        metadata = self.last_event_metadata.copy()
        metrics[metric_name] = metric_value
        metadata[f"threshold_{metric_name}"] = threshold
        body = {
            "event_id": self.last_event_id,
            "metadata": metadata,
            "metrics": metrics,
        }
        res = requests_retry_session().put(
            url=f"{self._base_url}/events",
            headers=self._headers,
            json=body,
        )
        if res.status_code == 200:
            self.last_event_metrics = metrics
            self.last_event_metadata = metadata

    def _start_new_session(self, inputs):
        session_body = {
            "project": self.project,
            "source": self.source,
            "session_id": self.session_id,
            "session_name": self.name,
            "user_properties": self.user_properties,
            "metadata": self.metadata,
            "inputs": inputs,
        }
        requests_retry_session().post(
            url=f"{self._base_url}/session/start",
            headers=self._headers,
            json=session_body,
        )

    def _persist_run(self, run: Run) -> None:
        """Persist a run."""
        try:
            logs = self._convert_run_to_logs(run=run)
            if self.project is not None:
                logs[0].project = self.project
            self._post_trace(logs=logs)
        except Exception as error:
            logging.warning(f"Failed to persist run: {error}")
            if self.verbose:
                traceback.print_exc()

    def _convert_run_to_logs(
        self, run: Run, parent_log: Optional[Log] = None
    ) -> List[Log]:
        logs: List[Log] = []
        duration = (run.end_time - run.start_time) / timedelta(milliseconds=1)
        metadata = {"langchain_trace_id": str(run.id), **run.extra}
        run_type = run.serialized.get("_type")
        if run.run_type == "chain":
            if run_type == LLM_CHAIN_TYPE:
                child_llm_runs = [
                    child_run
                    for child_run in run.child_runs
                    if child_run.run_type == "llm"
                ]
                run.child_runs = [
                    child_run
                    for child_run in run.child_runs
                    if child_run.run_type != "llm"
                ]
                inputs = {
                    input_: value
                    for input_, value in run.inputs.items()
                    if input_ in run.serialized["prompt"]["input_variables"]
                }
                for llm_run in child_llm_runs:
                    logs += self._convert_llm_run_to_log(
                        llm_run=llm_run,
                        event_name=run.name,
                        inputs=[inputs],
                        prompt_details=run.serialized["prompt"],
                        duration=duration,
                        parent_log=parent_log,
                    )
            elif run_type == AGENT_CHAIN_TYPE:
                logs.append(
                    self._convert_agent_run_to_log(
                        run=run,
                        event_name=run.name,
                        duration=duration,
                        metadata=metadata,
                        parent_log=parent_log,
                    )
                )
            else:
                logs.append(
                    self._convert_generic_chain_run_to_log(
                        run=run,
                        event_name=run.name,
                        duration=duration,
                        metadata=metadata,
                        parent_log=parent_log,
                    )
                )
        elif run.run_type in ["tool", "retriever"]:
            logs.append(
                self._convert_tool_run_to_log(
                    run=run,
                    event_name=run.name,
                    duration=duration,
                    metadata=metadata,
                    parent_log=parent_log,
                )
            )
        elif run.run_type == "llm":
            logs += self._convert_llm_run_to_log(
                llm_run=run,
                event_name=run.name,
                inputs=[self._convert_chain_inputs_to_text(run.inputs)],
                duration=duration,
                parent_log=parent_log,
            )
        else:
            raise NotImplementedError

        if parent_log is not None:
            if parent_log.children is None:
                parent_log.children = logs
            else:
                parent_log.children += logs
        child_runs = sorted(
            run.child_runs,
            key=lambda x: x.start_time,
        )
        for run in child_runs:
            # Assume the final log at this step is the parent
            _ = self._convert_run_to_logs(
                run=run,
                parent_log=logs[-1],
            )
        return logs

    def _post_trace(self, logs: List[Log]) -> None:
        """Post a trace to the HoneyHive API"""
        root_log = logs[0].dict()
        self.final_outputs = root_log["outputs"]
        self.session_id = str(uuid.uuid4())
        self._crawl(root_log, self.session_id)
        self._start_new_session(root_log["inputs"])
        trace_response = requests_retry_session().post(
            url=f"{self._base_url}/session/{self.session_id}/traces",
            json={"logs": [root_log]},
            headers=self._headers,
        )
        if trace_response.status_code != 200:
            raise TracerException(
                f"Failed to post trace to HoneyHive with status code {trace_response.status_code}"
            )
        requests_retry_session().put(
            url=f"{self._base_url}/events",
            json={"event_id": self.session_id, "outputs": self.final_outputs},
            headers=self._headers,
        )
        if self.eval_info:
            try:
                if "run_id" in self.eval_info:
                    run_res = requests_retry_session().get(
                        url=f"{self._base_url}/runs/{self.eval_info['run_id']}",
                        headers=self._headers,
                    )
                    event_ids = run_res.json()["evaluation"]["event_ids"]
                    event_ids.append(self.session_id)
                    datapoint_ids = run_res.json()["evaluation"]["event_ids"]
                    if "datapoint_id" in self.eval_info:
                        datapoint_ids.append(self.eval_info["datapoint_id"])
                    requests_retry_session().put(
                        url=f"{self._base_url}/runs/{self.eval_info['run_id']}",
                        json={"event_ids": event_ids, "datapoint_ids": datapoint_ids},
                        headers=self._headers,
                    )
                else:
                    body = {
                        "event_ids": [self.session_id],
                        "dataset_id": self.eval_info["dataset_id"],
                        "datapoint_ids": self.eval_info["datapoint_ids"],
                        "project": self.eval_info["project_id"],
                        "status": "completed",
                        "name": self.eval_info["run_name"],
                    }
                    if "config" in root_log:
                        body["configuration"] = root_log["config"]
                    run_res = requests_retry_session().post(
                        url=f"{self._base_url}/runs",
                        headers=self._headers,
                        json=body,
                    )
                    run_id = run_res.json()["run_id"]
                    self.eval_info["run_id"] = run_id
            except Exception as error:
                logging.warning(f"Failed to process eval: {error}")
                if self.verbose:
                    traceback.print_exc()

    def _crawl(self, trace, session_id) -> None:
        def crawl(node):
            if node is None:
                return
            node["session_id"] = session_id
            self.last_event_id = node["event_id"]
            self.last_event_metrics = node.get("metrics", {})
            self.last_event_metadata = node.get("metadata", {})
            self.final_outputs = node["outputs"]
            if node["children"]:
                for child in node["children"]:
                    child["parent_id"] = node["event_id"]
                    crawl(child)

        crawl(trace)

    @staticmethod
    def _convert_chain_outputs_to_text(outputs: Dict[str, Any]) -> str:
        """Convert LC dictionary outputs to a string required by the HH API."""
        output = ""
        for key, value in outputs.items():
            # TODO: check are there other data types to deal with
            if isinstance(value, list):
                value = ", ".join([str(v) for v in value])
            output += f"{key}:\n{value}\n"
        return output

    @staticmethod
    def _convert_chain_inputs_to_text(inputs: Dict[str, Any]) -> Dict[str, str]:
        """Convert LC list inputs to a string required by the HH API."""
        new_inputs = {}
        for key, value in inputs.items():
            # TODO: check are there other data types to deal with
            if isinstance(value, list):
                value = ", ".join([str(v) for v in value])
            new_inputs[key] = value
        return new_inputs

    @staticmethod
    def _convert_provider_parameters(
        llm_type: str, params: Dict[str, Any]
    ) -> Tuple[Dict[str, Any], Dict[str, Any]]:
        """Convert provider specific parameters to the HH common model parameters.

        Return a tuple of HoneyHive parameters and any parameters not mapped.
        """
        if not params:
            return {}, {}
        params = copy.deepcopy(params)
        # use startswith because LC has patterns 'openai', 'openai-chat' as _llm_types
        # TODO: Don't want to rely on string ops for getting provider,
        #  suggest to have separate 'provider' and 'mode' (complete or chat) instead
        #  of just _llm_type
        if llm_type.startswith("openai"):
            provider = "openai"
            mapping = {
                "model_name": "model",
                "temperature": "temperature",
                "max_tokens": "max_tokens",
                "n": "num_samples",
                "top_p": "top_p",
                "presence_penalty": "presence_penalty",
                "frequency_penalty": "frequency_penalty",
                "stop": "stop",
            }
        elif llm_type == "anthropic":
            provider = "anthropic"
            mapping = {
                "provider": "anthropic",
                "model_name": "model",
                "max_tokens_to_sample": "max_tokens",
                "temperature": "temperature",
                "top_k": "top_k",
                "top_p": "top_p",
                "stop_sequence": "stop",
            }
        else:
            raise NotImplementedError

        mapped_params = {
            hl_param: params.pop(provider_param, None)
            for provider_param, hl_param in mapping.items()
        }
        hl_params = {
            **mapped_params,
            "provider": provider,
        }
        return hl_params, params

    def _convert_agent_run_to_log(
        self,
        run: Run,
        event_name: str,
        duration: float,
        metadata: Dict[str, Any],
        parent_log: Optional[Log],
    ) -> Log:
        """Converts LC agent chain run to HH log."""
        if parent_log is not None:
            parent_id = parent_log.event_id
        else:
            parent_id = None
        config = AgentConfig(
            agent_class=run.serialized["agent"]["_type"],
            tools=[
                ToolConfig(
                    name=tool["name"],
                    description=tool["description"],
                    source=self._get_tool_source_from_function(
                        tool["name"], tool["func"]
                    )
                    if "func" in tool
                    else None,
                )
                for tool in run.serialized["tools"]
            ],
            model=self._create_model_config_from_params(
                params=run.serialized["agent"]["llm_chain"]["llm"],
                prompt_details=run.serialized["agent"]["llm_chain"]["prompt"],
            ),
            other=AgentOther(
                max_iterations=run.serialized["max_iterations"],
                stop=run.serialized["early_stopping_method"],
                output_parser=run.serialized["agent"]["output_parser"]["_type"],
            ),
        )
        return Log(
            source=self.source,
            project=self.project,
            children=None,
            event_name=event_name,
            event_type="chain",
            event_id=str(uuid.uuid4()),
            parent_id=parent_id,
            config=config,
            inputs=self._convert_chain_inputs_to_text(inputs=run.inputs),
            error=run.error,
            outputs={"text": self._convert_chain_outputs_to_text(run.outputs)}
            if run.outputs is not None
            else None,
            start_time=int(run.start_time.timestamp() * 1000000),
            end_time=int(run.end_time.timestamp() * 1000000),
            duration=duration,
            metadata=metadata,
        )

    def _convert_generic_chain_run_to_log(
        self,
        run: Run,
        event_name: str,
        duration: float,
        metadata: Dict[str, Any],
        parent_log: Optional[Log],
    ) -> Log:
        """Converts LC generic chain run to HH log."""
        if parent_log is not None:
            parent_id = parent_log.event_id
        else:
            parent_id = None
        config = Config(name=event_name)
        return Log(
            source=self.source,
            project=self.project,
            event_name=event_name,
            event_type="generic",
            config=config,
            event_id=str(uuid.uuid4()),
            parent_id=parent_id,
            inputs=self._convert_chain_inputs_to_text(inputs=run.inputs),
            error=run.error,
            children=None,
            outputs={"text": self._convert_chain_outputs_to_text(run.outputs)}
            if run.outputs is not None
            else None,
            end_time=int(run.end_time.timestamp() * 1000000),
            start_time=int(run.start_time.timestamp() * 1000000),
            duration=duration,
            metadata=metadata,
        )

    @staticmethod
    def _get_tool_source_from_function(tool_name: str, tool_function: Callable) -> str:
        """Get the source code for a tool function."""
        try:
            return inspect.getsource(tool_function)
        except Exception as _:
            logging.info(f"Failed to get source for tool {tool_name}")
            traceback.print_exc()

    def _convert_tool_run_to_log(
        self,
        run: Run,
        event_name: str,
        duration: float,
        metadata: Dict[str, Any],
        parent_log: Optional[Log],
    ) -> Log:
        """Converts LC tool chain run to HH log."""
        if parent_log is not None:
            parent_id = parent_log.event_id
        else:
            parent_id = None
        description = (
            run.serialized["description"] if run.run_type != "retriever" else None
        )
        return Log(
            source=self.source,
            project=self.project,
            children=None,
            event_name=event_name,
            event_type="tool",
            event_id=str(uuid.uuid4()),
            parent_id=parent_id,
            config=ToolConfig(
                name=event_name,
                description=description,
                source=self._get_tool_source_from_function(
                    event_name, run.serialized["func"]
                )
                if "func" in run.serialized
                else None,
                other=run.extra,
            ),
            inputs=run.inputs,
            outputs={"text": self._convert_chain_outputs_to_text(run.outputs)}
            if run.outputs is not None
            else None,
            error=run.error,
            metadata=metadata,
            end_time=int(run.end_time.timestamp() * 1000000),
            start_time=int(run.start_time.timestamp() * 1000000),
            duration=duration,
        )

    @staticmethod
    def _convert_template_to_hl_syntax(template: str, template_format: str) -> str:
        """Converts LC template to HH double curly bracket syntax"""
        if template_format == "f-string":
            # match the f-string syntax with single curly brackets
            pattern = r"{([^{}]+)}"

            # replaces f-string syntax with Jinja2 syntax
            def replace(match):
                return "{{{{ {} }}}}".format(match.group(1))

            return re.sub(pattern, replace, template)
        elif template_format == "jinja2":
            # jinja2 already uses double curly brackets
            return template
        else:
            logging.info(f"Unknown template format: {template_format}")
            return template

    def _create_model_config_from_params(
        self,
        params: Dict[str, Any],
        prompt_details: Optional[Dict[Any]] = None,
    ) -> ModelConfig:
        """Creates HH model configuration from LC parameters."""
        hl_params, other_params = self._convert_provider_parameters(
            llm_type=params.pop("_type"),
            params=params,
        )
        if prompt_details is not None:
            if "messages" in prompt_details:
                return ModelConfig(
                    chat_template=[
                        ChatMessage(
                            content=self._convert_template_to_hl_syntax(
                                template=message["prompt"]["template"],
                                template_format=message["prompt"]["template_format"],
                            ),
                            role=ROLE_MAPPING[message["role"]],
                        )
                        for message in prompt_details["messages"]
                    ],
                    endpoints="chat",
                    **hl_params,
                )
            else:
                return ModelConfig(
                    prompt_template=self._convert_template_to_hl_syntax(
                        template=prompt_details["template"],
                        template_format=prompt_details["template_format"],
                    ),
                    endpoint="complete",
                    **hl_params,
                )
        else:
            return ModelConfig(**hl_params)

    def _convert_llm_run_to_log(
        self,
        llm_run: Run,
        event_name: str,
        inputs: List[Dict[str, str]],
        duration: float,
        parent_log: Optional[Log],
        prompt_details: Optional[Dict[Any]] = None,
    ) -> List[Log]:
        if parent_log is not None:
            parent_id = parent_log.event_id
        else:
            parent_id = None
        """Converts LC llm run to HH log."""
        config = self._create_model_config_from_params(
            params=llm_run.extra.pop("invocation_params", {}),
            prompt_details=prompt_details,
        )
        metadata = {"langchain_run_id": str(llm_run.id), **llm_run.extra}
        if "generations" in llm_run.outputs:
            logs = []
            # response.generations is a list (# inputs) of lists (# samples)
            for i, generations in enumerate(llm_run.outputs["generations"]):
                for generation in generations:
                    if "message" in generation:
                        if "content" in generation["message"]:
                            output = generation["message"]["content"]
                        elif (
                            "kwargs" in generation["message"]
                            and "content" in generation["message"]["kwargs"]
                        ):
                            output = generation["message"]["kwargs"]["content"]
                        else:
                            raise NotImplementedError
                    elif "text" in generation:
                        output = generation["text"]
                    else:
                        raise NotImplementedError
                    logs.append(
                        # TODO: add token usage
                        # TODO: chat serialization doesn't include messages array
                        Log(
                            source=self.source,
                            project=self.project,
                            children=None,
                            event_name=event_name,
                            event_type="model",
                            event_id=str(uuid.uuid4()),
                            parent_id=parent_id,
                            config=config,
                            inputs=inputs[i],
                            outputs={"text": output},
                            start_time=int(llm_run.start_time.timestamp() * 1000000),
                            end_time=int(llm_run.end_time.timestamp() * 1000000),
                            duration=duration,
                            metadata=metadata,
                        )
                    )
        else:
            logs = [
                Log(
                    source=self.source,
                    project=self.project,
                    children=None,
                    event_name=event_name,
                    event_type="model",
                    event_id=str(uuid.uuid4()),
                    parent_id=parent_id,
                    config=config,
                    inputs=inputs,
                    error=llm_run.error,
                    start_time=int(llm_run.start_time.timestamp() * 1000000),
                    end_time=int(llm_run.end_time.timestamp() * 1000000),
                    duration=duration,
                    metadata=metadata,
                )
            ]
        return logs

    def load_session(self, session_name: str) -> TracerSession:
        """Load a tracing session and set it as the Tracer's session."""
        # LCs session concept is a collection of runs, not required by HH.
        return TracerSession(id=-1)

    def load_default_session(self) -> TracerSession:
        """Load the default tracing session and set it as the Tracer's session."""
        # LCs session concept is a collection of runs, not required by HH.
        return TracerSession(id=-1)


class ChatRole(str, Enum):
    user = "user"
    assistant = "assistant"
    system = "system"


class ChatMessage(BaseModel):
    role: ChatRole
    content: str
    name: Optional[str] = None


class Config(BaseModel):
    type: str = "generic"
    name: Optional[str] = None
    description: Optional[str] = None
    model_config = ConfigDict(protected_namespaces=())


class LLMConfig(Config):
    type: str = "model"
    model_name: Optional[str] = None
    api_base: Optional[str] = None
    class_name: Optional[str] = None
    api_version: Optional[str] = None


class ModelConfig(Config):
    provider: str = Field(
        title="Model provider",
        description="The company providing the underlying model service.",
    )
    endpoint: Optional[str] = Field(
        title="Provider endpoint",
        description="Which of the providers model endpoints to use. "
        "For example Complete or Edit.",
        default=None,
    )
    model: str = Field(
        title="Model instance used",
        description="What model instance to use for the generation. "
        "e.g. text-davinci-002.",
    )
    prompt_template: Optional[str] = Field(
        title="Prompt template",
        description="Prompt template that will take your specified inputs to form "
        "your final request to the provider model. "
        "Input variables within the prompt template should be specified "
        "with double curly bracket syntax: {{INPUT_NAME}}.",
        default=None,
    )
    chat_template: Optional[List[ChatMessage]] = Field(
        title="Chat template",
        description="Messages prepended to the list of messages sent to the provider. "
        "These messages that will take your specified inputs to form "
        "your final request to the provider model. ",
        default=None,
    )
    temperature: Optional[float] = Field(
        title="Sampling temperature",
        description="What sampling temperature to use when making a generation. "
        "Higher values means the model will be more creative.",
        default=1,
    )
    max_tokens: Optional[int] = Field(
        title="Maximum tokens",
        description="The maximum number of tokens to generate. "
        "Provide max_tokens=-1 to dynamically calculate the maximum number of tokens "
        "to generate given the length of the prompt",
        default=-1,
    )
    top_p: Optional[float] = Field(
        title="Top p probability mass",
        description="An alternative to sampling with temperature, "
        "called nucleus sampling, where the model considers the results "
        "of the tokens with top_p probability mass.",
        default=1,
    )
    stop: Optional[Union[str, List[str]]] = Field(
        title="Stop sequence(s)",
        description="The string (or list of strings) after which the model will stop "
        "generating. The returned text will not contain the stop sequence.",
        default=None,
    )
    presence_penalty: Optional[float] = Field(
        title="Penalize tokens on whether present.",
        description="Number between -2.0 and 2.0. Positive values penalize new tokens "
        "based on whether they appear in the generation so far.",
        default=0,
    )
    frequency_penalty: Optional[float] = Field(
        title="Penalize tokens on whether frequent.",
        description="Number between -2.0 and 2.0. Positive values penalize new tokens "
        "based on how frequently they appear in the generation so far.",
        default=0,
    )
    other: Optional[Dict[str, Any]] = Field(
        title="Other provider parameters",
        description="Other parameter values to be passed to the provider call.",
        default={},
    )
    type: str = "model"


class ToolConfig(Config):
    source: Optional[str] = Field(
        title="Tool source",
        description="The source code for the tool.",
        default=None,
    )
    other: Optional[Dict[str, Any]] = Field(
        title="Other tool parameters",
        description="Other parameter values that uniquely identify the tool.",
        default={},
    )
    type: str = "tool"


class AgentOther(BaseModel):
    max_iterations: int
    stop: Optional[Union[str, List[str]]]
    output_parser: Optional[str]


class AgentConfig(Config):
    agent_class: str
    tools: List[ToolConfig]
    model: ModelConfig
    other: AgentOther
    type: str = "agent"


class Log(BaseModel):
    project: Optional[str] = Field(
        title="Project name",
        description="The name of the project under which you're running the tracing",
    )
    event_id: Optional[str] = Field(
        title="Event ID",
        description="A unique ID for the event",
    )
    parent_id: Optional[str] = Field(
        title="Parent ID",
        description="Event ID of the parent event",
    )
    event_type: str = Field(
        title="Event type",
        description="Event type: Model, Tool, Chain or Generic",
    )
    event_name: str = Field(
        title="Function name",
        description="Function name. If it does not exist, a new function will be "
        "created.",
    )
    config: Union[Config, ModelConfig, ToolConfig, AgentConfig] = Field(
        title="Config",
        description="The config used for a specific step in the chain.",
    )
    inputs: Dict[str, Any] = Field(
        title="Project input data",
        description="List of (name, value) pairs for the inputs used by your prompt "
        "template, or directly by your project.",
    )
    outputs: Optional[Dict[str, Any]] = Field(
        title="Model output",
        description="Generated output from your model for the provided inputs. Can be None if error encountered.",
    )
    children: Optional[List[Log]]
    user_properties: Optional[Dict[str, Any]] = Field(
        title="User properties",
        description="Metadata associated with the user",
        default={},
    )
    metadata: Optional[Dict[str, Any]] = Field(
        title="Event properties",
        description="Metadata associated with the event",
        default={},
    )
    source: Optional[str] = Field(
        title="Source of generation",
        description="What was source of the model used for this generation? "
        "e.g. langchain",
        default="langchain",
    )
    start_time: Optional[int] = Field(
        title="Start time",
        description="The time in epoch microseconds the event started",
    )
    end_time: Optional[int] = Field(
        title="End time",
        description="How time in epoch microseconds the event ended",
    )
    duration: Optional[float] = Field(
        title="Duration (in ms)",
        description="The duration of the event in milliseconds",
    )
    error: Optional[str] = Field(
        title="Log error",
        description="Captures error thrown by model.",
        default=None,
    )
    metrics: Optional[Dict[str, float]] = Field(
        title="Metrics",
        description="Metrics associated with the event",
        default={},
    )
    feedback: Optional[Dict[str, Any]] = Field(
        title="Feedback",
        description="Feedback associated with the event",
        default={},
    )


def log_to_dict(log):
    if isinstance(log, Log):
        return {
            "project": log.project,
            "event_id": log.event_id,
            "parent_id": log.parent_id,
            "event_type": log.event_type,
            "event_name": log.event_name,
            "config": config_to_dict(log.config),
            "inputs": recursive_serialize(log.inputs),
            "outputs": recursive_serialize(log.outputs),
            "children": [log_to_dict(c) for c in log.children]
            if log.children
            else None,
            "start_time": log.start_time,
            "end_time": log.end_time,
            "user_properties": log.user_properties,
            "metadata": log.metadata,
            "source": log.source,
            "error": log.error,
            "metrics": log.metrics,
            "feedback": log.feedback,
            "duration": log.duration,
        }


def recursive_serialize(item):
    # Base case: if the item is a dictionary, process its values
    if isinstance(item, dict):
        return {k: recursive_serialize(v) for k, v in item.items()}

    # If the item is a list or tuple, process its elements
    elif isinstance(item, (list, tuple)):
        return [recursive_serialize(e) for e in item]

    # If the item has a to_dict method, call it
    elif hasattr(item, "to_dict"):
        return recursive_serialize(item.to_dict())

    # Try to serialize the item using the default JSON serialization
    try:
        json.dumps(item)
        return item
    except TypeError:
        # If the item is not JSON serializable, return its string representation
        return str(item)


def config_to_dict(config):
    # Handle config field
    if config:
        return config.dict()
    return None


def requests_retry_session(
    retries=8,
    backoff_factor=0.3,
    status_forcelist=(400, 500, 502, 503, 504),
    session=None,
    jitter_base=0.1,
):
    """
    Creates a requests session with retry logic including exponential backoff with jitter.

    Args:
        retries (int): Number of retries.
        backoff_factor (float): A base factor to apply for exponential backoff.
        status_forcelist (tuple): A set of HTTP status codes that we should force a retry on.
        session (requests.Session, optional): Use an existing session if provided, otherwise create a new one.

    Returns:
        requests.Session: A requests session configured with retry logic including jitter.
    """
    session = session or requests.Session()
    retry = Retry(
        total=retries,
        read=retries,
        connect=retries,
        backoff_factor=backoff_factor,
        status_forcelist=status_forcelist,
        allowed_methods=["GET", "PUT", "POST"],
        raise_on_status=False,
    )

    def backoff_with_jitter(retry, *args, **kwargs):
        # Calculate the normal backoff
        backoff_value = retry.get_backoff_time()
        # Apply jitter by randomizing the backoff time
        jittered_backoff = backoff_value + random.uniform(0, jitter_base)
        return jittered_backoff

    # Override the backoff method
    retry.get_backoff_time = backoff_with_jitter

    adapter = HTTPAdapter(max_retries=retry)
    session.mount("http://", adapter)
    session.mount("https://", adapter)
    return session


__all__ = ["HoneyHiveLangChainTracer"]



import openai
import os
import honeyhive
import time
import uuid
from honeyhive.models import components, operations
from honeyhive.tracer import HoneyHiveTracer
from honeyhive.tracer.custom import trace
from llama_index.core import VectorStoreIndex
from llama_index.readers.web import SimpleWebPageReader

session_name = f"HoneyHive Tracer Test {str(uuid.uuid4())}"
hhtracer = HoneyHiveTracer.init(
    server_url=os.environ["HH_API_URL"],
    api_key=os.environ["HH_API_KEY"],
    project=os.environ["HH_PROJECT"],
    source="HoneyHive Tracer Test",
    session_name=session_name,
)
sdk = honeyhive.HoneyHive(
    bearer_auth=os.environ["HH_API_KEY"], server_url=os.environ["HH_API_URL"]
)


@trace(config={"thing": "stuff"}, metadata={"meta_thing": 42})
def run_tracer_enriched(input, prompt_template):
    openai.api_key = os.environ["OPENAI_API_KEY"]

    documents = SimpleWebPageReader(html_to_text=True).load_data(
        ["http://paulgraham.com/worked.html"]
    )

    index = VectorStoreIndex.from_documents(documents)

    query_engine = index.as_query_engine()
    response = query_engine.query(prompt_template["prompt"][0]["content"])
    return response


def run_tracer():
    openai.api_key = os.environ["OPENAI_API_KEY"]

    documents = SimpleWebPageReader(html_to_text=True).load_data(
        ["http://paulgraham.com/worked.html"]
    )

    index = VectorStoreIndex.from_documents(documents)

    query_engine = index.as_query_engine()
    response = query_engine.query("What did the author do growing up?")
    return response

def test_tracer():
    run_tracer_enriched(
        {"a": 3, "b": [1, 2, 3], "c": {"d": [4, 5, 6]}},
        {
            "template": [
                {"role": "user", "content": "What did {{subject}} do {{participial}}?"}
            ],
            "prompt": [
                {"role": "user", "content": "What did the author do growing up?"}
            ],
        },
    )

    # Get session
    time.sleep(10)
    req = operations.GetEventsRequestBody(
        project=os.environ["HH_PROJECT"],
        filters=[
            components.EventFilter(
                field="event_name",
                value=session_name,
                operator=components.Operator.IS,
            ),
            components.EventFilter(
                field="event_type",
                value="session",
                operator=components.Operator.IS,
            ),
        ],
    )
    res = sdk.events.get_events(request=req)
    assert res.status_code == 200
    assert res.object is not None
    assert len(res.object.events) == 1

    session_id = res.object.events[0].session_id
    req = operations.GetEventsRequestBody(
        project=os.environ["HH_PROJECT"],
        filters=[
            components.EventFilter(
                field="session_id",
                value=session_id,
                operator=components.Operator.IS,
            ),
        ],
    )
    res = sdk.events.get_events(request=req)
    assert res.status_code == 200
    assert res.object is not None
    assert len(res.object.events) > 1

    req = operations.GetEventsRequestBody(
        project=os.environ["HH_PROJECT"],
        filters=[
            components.EventFilter(
                field="event_name",
                value="run_tracer_enriched",
                operator=components.Operator.IS,
            ),
            components.EventFilter(
                field="session_id",
                value=session_id,
                operator=components.Operator.IS,
            ),
        ],
    )
    res = sdk.events.get_events(request=req)
    assert res.status_code == 200
    assert res.object is not None
    assert len(res.object.events) == 1
    event = res.object.events[0]
    assert event.inputs is not None
    assert "_params_" in event.inputs
    assert event.outputs is not None
    assert "result" in event.outputs
    assert event.config.get("thing") == "stuff"
    assert event.metadata.get("meta_thing") == 42

    # Run it a second time in a new session
    HoneyHiveTracer.init(
        server_url=os.environ["HH_API_URL"],
        api_key=os.environ["HH_API_KEY"],
        project=os.environ["HH_PROJECT"],
        source="HoneyHive Tracer Test",
        session_name=session_name,
    )
    run_tracer()

    time.sleep(10)

    req = operations.GetEventsRequestBody(
        project=os.environ["HH_PROJECT"],
        filters=[
            components.EventFilter(
                field="event_name",
                value=session_name,
                operator=components.Operator.IS,
            ),
            components.EventFilter(
                field="event_type",
                value="session",
                operator=components.Operator.IS,
            ),
        ],
    )
    res = sdk.events.get_events(request=req)
    assert res.status_code == 200
    assert res.object is not None
    assert len(res.object.events) == 2

    new_session_id = None
    for event in res.object.events:
        if event.session_id is not None and event.session_id != session_id:
            new_session_id = event.session_id
    assert new_session_id is not None

    req = operations.GetEventsRequestBody(
        project=os.environ["HH_PROJECT"],
        filters=[
            components.EventFilter(
                field="session_id",
                value=session_id,
                operator=components.Operator.IS,
            ),
            components.EventFilter(
                field="event_name",
                value="OpenAI.task",
                operator=components.Operator.IS,
            ),
        ],
    )
    res = sdk.events.get_events(request=req)
    assert res.status_code == 200
    assert res.object is not None
    assert len(res.object.events) == 1

    req = operations.GetEventsRequestBody(
        project=os.environ["HH_PROJECT"],
        filters=[
            components.EventFilter(
                field="session_id",
                value=new_session_id,
                operator=components.Operator.IS,
            ),
        ],
    )
    res = sdk.events.get_events(request=req)
    assert res.status_code == 200
    assert res.object is not None
    assert len(res.object.events) > 1

def test_tracer_metadata_update():
    run_tracer()
    time.sleep(10)

    hhtracer.enrich_session(metadata={"test": "value"})
    time.sleep(10)

    session_id = hhtracer.session_id
    req = operations.GetEventsRequestBody(
        project=os.environ["HH_PROJECT"],
        filters=[
            components.EventFilter(
                field="session_id",
                value=session_id,
                operator=components.Operator.IS,
            ),
            components.EventFilter(
                field="event_type",
                value="session",
                operator=components.Operator.IS,
            ),
        ],
    )
    res = sdk.events.get_events(request=req)
    assert res.status_code == 200
    assert res.object is not None
    assert len(res.object.events) == 1

    logged_event = res.object.events[0]
    assert "test" in logged_event.metadata
    assert logged_event.metadata["test"] == "value"

def test_tracer_feedback_update():
    run_tracer()
    time.sleep(10)

    hhtracer.enrich_session(feedback={"comment": "test feedback"})
    time.sleep(10)

    session_id = hhtracer.session_id
    req = operations.GetEventsRequestBody(
        project=os.environ["HH_PROJECT"],
        filters=[
            components.EventFilter(
                field="session_id",
                value=session_id,
                operator=components.Operator.IS,
            ),
            components.EventFilter(
                field="event_type",
                value="session",
                operator=components.Operator.IS,
            ),
        ],
    )
    res = sdk.events.get_events(request=req)
    assert res.status_code == 200
    assert res.object is not None
    assert len(res.object.events) == 1

    logged_event = res.object.events[0]
    assert logged_event.feedback == {"comment": "test feedback"}


def test_tracer_evaluator_update():
    run_tracer()
    time.sleep(10)

    hhtracer.enrich_session(metrics={"tps": 1.78})
    time.sleep(10)

    session_id = hhtracer.session_id
    req = operations.GetEventsRequestBody(
        project=os.environ["HH_PROJECT"],
        filters=[
            components.EventFilter(
                field="session_id",
                value=session_id,
                operator=components.Operator.IS,
            ),
            components.EventFilter(
                field="event_type",
                value="session",
                operator=components.Operator.IS,
            ),
        ],
    )
    res = sdk.events.get_events(request=req)
    assert res.status_code == 200
    assert res.object is not None
    assert len(res.object.events) == 1

    logged_event = res.object.events[0]
    assert logged_event.metrics == {"tps": 1.78}


def test_distributed_tracing():
    pre_existing_session_id = "fb0a4180-c998-45a6-ba0a-b19bf46e966b"
    req = operations.GetEventsRequestBody(
        project=os.environ["HH_PROJECT"],
        filters=[
            components.EventFilter(
                field="session_id",
                value=pre_existing_session_id,
                operator=components.Operator.IS,
            ),
        ],
        limit=7500,
    )
    res = sdk.events.get_events(request=req)
    assert res.status_code == 200
    assert res.object is not None
    assert len(res.object.events) > 1
    prev_event_count = len(res.object.events)

    hhtracer.init_from_session_id(
        server_url=os.environ["HH_API_URL"],
        api_key=os.environ["HH_API_KEY"],
        session_id=pre_existing_session_id,
    )
    run_tracer()

    time.sleep(15)
    req = operations.GetEventsRequestBody(
        project=os.environ["HH_PROJECT"],
        filters=[
            components.EventFilter(
                field="session_id",
                value=pre_existing_session_id,
                operator=components.Operator.IS,
            ),
        ],
        limit=7500,
    )
    res = sdk.events.get_events(request=req)
    assert res.status_code == 200
    assert res.object is not None
    assert len(res.object.events) > prev_event_count
